##
# File:    OeSearchIndexUtilsTests.py
# Author:  J. Westbrook
# Date:    1-Oct-2019
# Version: 0.001
#
# Update:
#
#
##
"""
Tests for search modes using source molecular definitions coming from a search index.

"""
import logging
import os
import platform
import resource
import time
import unittest

from importlib.metadata import version as get_package_version
from rcsb.utils.chem.ChemCompIndexProvider import ChemCompIndexProvider
from rcsb.utils.chem.OeDepictAlign import OeDepictMCSAlignPage
from rcsb.utils.chem.OeIoUtils import OeIoUtils
from rcsb.utils.chem.OeSearchMoleculeProvider import OeSearchMoleculeProvider
from rcsb.utils.chem.OeSearchUtils import OeSearchUtils

__version__ = get_package_version("rcsb.utils.chem")

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()
logger.setLevel(logging.INFO)


class OeSearchIndexUtilsTests(unittest.TestCase):
    skipFlag = True

    def setUp(self):
        self.__useCacheFlag = False
        self.__workPath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif")
        self.__birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif")
        # self.__fpTypeList = ["TREE", "PATH", "MACCS", "CIRCULAR", "LINGO"]
        self.__fpTypeCuttoffList = [("TREE", 0.6), ("PATH", 0.6), ("MACCS", 0.9), ("CIRCULAR", 0.6), ("LINGO", 0.9)]
        self.__screenTypeList = ["SMARTS"]
        self.__numProc = 1
        self.__minCount = 500
        self.__startTime = time.time()
        #
        # self.__buildTypeList = ["oe-iso-smiles", "oe-smiles", "acdlabs-smiles", "cactvs-iso-smiles", "cactvs-smiles", "inchi"]
        self.__buildTypeList = ["oe-iso-smiles", "oe-smiles", "cactvs-iso-smiles", "cactvs-smiles", "inchi"]
        self.__numMols = 31
        self.__myKwargs = {
            "ccUrlTarget": self.__ccUrlTarget,
            "birdUrlTarget": self.__birdUrlTarget,
            "cachePath": self.__cachePath,
            "useCache": self.__useCacheFlag,
            "ccFileNamePrefix": "cc-abbrev",
            "oeFileNamePrefix": "oe-abbrev",
            "limitPerceptions": False,
            "minCount": 20,
            "maxFpResults": 50,
            "fpTypeCuttoffList": self.__fpTypeCuttoffList,
            "buildTypeList": self.__buildTypeList,
            "screenTypeList": self.__screenTypeList,
        }
        #
        logger.debug("Running tests on version %s", __version__)
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def __resultContains(self, ccId, matchResultList):
        for matchResult in matchResultList:
            if ccId in matchResult.ccId:
                return True
        return False

    def __getSearchDataProviders(self, **kwargs):
        oesmP = OeSearchMoleculeProvider(**kwargs)
        ok = oesmP.testCache()
        ccIdxP = ChemCompIndexProvider(**kwargs)
        ok = ccIdxP.testCache()
        self.assertTrue(ok)
        ccIdxD = ccIdxP.getIndex()
        return oesmP, ccIdxD

    def testSubStructureSearchExhaustiveAbbrev(self):
        """Exhaustive substructure search. (abbrev)"""
        return self.__exhaustiveSubStructureSearch(self.__numMols, **self.__myKwargs)

    def __exhaustiveSubStructureSearch(self, numMols, **kwargs):
        """Exhaustive substructure search."""
        try:
            limitPerceptions = kwargs.get("limitPerceptions", False)
            buildTypeList = kwargs.get("buildTypeList", ["oe-iso-smiles"])
            oesmP, ccIdxD = self.__getSearchDataProviders(**kwargs)
            oesU = OeSearchUtils(oesmP, fpTypeList=[])
            oeioU = OeIoUtils()
            #
            for ccId, ccD in list(ccIdxD.items())[:numMols]:
                for buildType in buildTypeList:
                    if buildType in ccD:
                        oeMol = oeioU.descriptorToMol(ccD[buildType], buildType, limitPerceptions=limitPerceptions, messageTag=ccId + ":" + buildType)
                        if not oeMol:
                            continue
                        # ----
                        startTime = time.time()
                        retStatus, mL = oesU.searchSubStructure(oeMol, matchOpts="graph-strict")
                        if not self.__resultContains(ccId, mL):
                            logger.info("%s match length %d build type %s in (%.4f seconds)", ccId, len(mL), buildType, time.time() - startTime)
                        self.assertTrue(retStatus)
                        self.assertTrue(self.__resultContains(ccId, mL))
                # ----
            return True
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()
        return False

    def testFingerprintSearchAbbrev(self):
        """Fingerprint search."""
        return self.__fingerPrintSearch(self.__numMols, **self.__myKwargs)

    def __fingerPrintSearch(self, numMols, **kwargs):
        maxFpResults = kwargs.get("maxFpResults", 50)
        limitPerceptions = kwargs.get("limitPerceptions", False)
        fpTypeCuttoffList = kwargs.get("fpTypeCuttoffList", [("TREE", 0.6)])
        buildTypeList = kwargs.get("buildTypeList", ["oe-iso-smiles"])
        #
        oesmP, ccIdxD = self.__getSearchDataProviders(**kwargs)
        oesU = OeSearchUtils(oesmP, fpTypeList=[tup[0] for tup in fpTypeCuttoffList])
        oeioU = OeIoUtils()
        # This will reload the oe binary cache.
        oeMol = oesmP.getMol("004")
        self.assertGreaterEqual(len(list(oeMol.GetAtoms())), 12)
        missedFpD = {}
        missedBuildD = {}
        numMols = min(len(ccIdxD), numMols) if numMols else len(ccIdxD)
        logger.info("Begin finger print search on %d molecules", numMols)
        # ----
        startTime = time.time()
        for ccId, ccD in list(ccIdxD.items())[:numMols]:
            for buildType in buildTypeList:
                if buildType in ccD:
                    oeMol = oeioU.descriptorToMol(ccD[buildType], buildType, limitPerceptions=limitPerceptions, messageTag=ccId + ":" + buildType)
                    if not oeMol:
                        continue
                    selfHit = False
                    for fpType, minFpScore in fpTypeCuttoffList:
                        retStatus, mL = oesU.searchFingerPrints(oeMol, fpType=fpType, minFpScore=minFpScore, maxFpResults=maxFpResults)
                        self.assertTrue(retStatus)
                        #
                        matchedSelf = self.__resultContains(ccId, mL)
                        selfHit = selfHit or matchedSelf
                        if not matchedSelf:
                            missedFpD.setdefault(ccId, []).append((buildType, fpType, len(mL)))
                    #
                    if not selfHit:
                        missedBuildD.setdefault(ccId, []).append(buildType)
        # ------
        for ccId, bTL in missedBuildD.items():
            logger.info("%s missed all fptypes:  buildtype list %r", ccId, bTL)

        if ccId in missedFpD:
            logger.info("%s unmatched by fpTypes %r", ccId, missedFpD[ccId])

        # ----
        logger.info("%s fingerprints search on %d in (%.4f seconds)", len(fpTypeCuttoffList), numMols, time.time() - startTime)
        # ----
        return True

    def testFingerPrintScoresAbbrev(self):
        """Fingerprint scores. (abbreviated)"""
        return self.__fingerPrintScores(self.__numMols, **self.__myKwargs)

    @unittest.skipIf(skipFlag, "Long troubleshooting test")
    def testFingerPrintScoresFull(self):
        """Fingerprint scores. (full)"""
        numMols = 300000
        myKwargs = {
            "cachePath": self.__cachePath,
            "useCache": True,
            "ccFileNamePrefix": "cc-full",
            "oeFileNamePrefix": "oe-full",
            "limitPerceptions": False,
            "minCount": 500,
            "maxFpResults": 50,
            "fpTypeCuttoffList": self.__fpTypeCuttoffList,
            "buildTypeList": self.__buildTypeList,
        }
        return self.__fingerPrintScores(numMols, **myKwargs)

    def __fingerPrintScores(self, numMols, **kwargs):
        maxFpResults = kwargs.get("maxResults", 50)
        limitPerceptions = kwargs.get("limitPerceptions", False)
        fpTypeCuttoffList = kwargs.get("fpTypeCuttoffList", [("TREE", 0.6)])
        buildTypeList = kwargs.get("buildTypeList", ["oe-iso-smiles"])
        doDisplay = kwargs.get("doDisplay", False)
        #
        oesmP, ccIdxD = self.__getSearchDataProviders(**kwargs)
        oesU = OeSearchUtils(oesmP, fpTypeList=[tup[0] for tup in fpTypeCuttoffList])
        oeioU = OeIoUtils()
        # This will reload the oe binary cache.
        oeMol = oesmP.getMol("004")
        self.assertGreaterEqual(len(list(oeMol.GetAtoms())), 12)
        #
        missedFpD = {}
        missedBuildD = {}
        numMols = min(len(ccIdxD), numMols) if numMols else len(ccIdxD)
        logger.info("Begin finger print score search on %d molecules", numMols)
        # ----
        startTime = time.time()
        for (
            ii,
            ccId,
        ) in enumerate(list(ccIdxD.keys())[:numMols]):
            ccD = ccIdxD[ccId]
            for buildType in buildTypeList:
                if buildType in ccD:
                    oeMol = oeioU.descriptorToMol(ccD[buildType], buildType, limitPerceptions=limitPerceptions, messageTag=ccId + ":" + buildType)
                    if not oeMol:
                        logger.debug("%s build failed for %s - skipping", ccId, buildType)
                        continue
                    maxHits = 0
                    minHits = maxFpResults
                    selfHit = False
                    #
                    startTime1 = time.time()
                    for fpType, minFpScore in fpTypeCuttoffList:
                        retStatus, mL = oesU.getFingerPrintScores(oeMol, fpType, minFpScore, maxFpResults)
                        self.assertTrue(retStatus)
                        logger.debug("%s fpType %r hits %d", ccId, fpType, len(mL))
                        maxHits = max(maxHits, len(mL))
                        minHits = min(minHits, len(mL))
                        matchedSelf = self.__resultContains(ccId, mL)
                        selfHit = selfHit or matchedSelf
                        if not matchedSelf:
                            missedFpD.setdefault(ccId, []).append((buildType, fpType, len(mL)))
                    #
                    if not selfHit:
                        missedBuildD.setdefault(ccId, []).append(buildType)
                    #
                    if maxHits < 1 or not selfHit:
                        logger.info("%s buildType %r min hits %d max hits %d (%.4f seconds)", ccId, buildType, minHits, maxHits, time.time() - startTime1)
                else:
                    logger.debug("%s missing descriptor %r", ccId, buildType)
            if ii % 100 == 0:
                logger.info("Completed %d of %d missed count %d", ii, numMols, len(missedBuildD))

        # ------
        for ccId, bTL in missedBuildD.items():
            logger.info("%s missed all fptypes:  buildtype list %r", ccId, bTL)

        if ccId in missedFpD:
            logger.info("%s unmatched by fpTypes %r", ccId, missedFpD[ccId])

        #
        if doDisplay:
            for ccId, bTL in missedBuildD.items():
                idxD = ccIdxD[ccId]
                if "oe-iso-smiles" in idxD:
                    for bT in bTL:
                        self.__displayAlignedDescriptorPair(ccId, idxD["oe-iso-smiles"], "oe-iso-smiles", idxD[bT], bT, title=None, limitPerceptions=True)

        logger.info("%s fingerprints search on %d in (%.4f seconds)", len(fpTypeCuttoffList), numMols, time.time() - startTime)
        return True

    def testSubStructureSearchWithFpAbbrev(self):
        """Substructure search with fingerprint prefilter. (abbreviated data)"""
        return self.__sssWithFingerPrintFromDescriptor(self.__numMols, **self.__myKwargs)

    @unittest.skipIf(skipFlag, "Long troubleshooting test")
    def testSubStructureSearchWithFpFull(self):
        """Substructure search with fingerprint prefilter. (full)"""
        numMols = 300000
        myKwargs = {
            "cachePath": self.__cachePath,
            "useCache": True,
            "ccFileNamePrefix": "cc-full",
            "oeFileNamePrefix": "oe-full",
            "limitPerceptions": False,
            "minCount": 500,
            "maxFpResults": 50,
            "fpTypeCuttoffList": self.__fpTypeCuttoffList,
            "buildTypeList": self.__buildTypeList,
        }
        return self.__sssWithFingerPrintFromDescriptor(numMols, **myKwargs)

    def __sssWithFingerPrintFromDescriptor(self, numMols, **kwargs):
        maxFpResults = kwargs.get("maxResults", 50)
        limitPerceptions = kwargs.get("limitPerceptions", False)
        fpTypeCuttoffList = kwargs.get("fpTypeCuttoffList", [("TREE", 0.6)])
        buildTypeList = kwargs.get("buildTypeList", ["oe-iso-smiles"])
        doDisplay = kwargs.get("doDisplay", False)
        #
        oesmP, ccIdxD = self.__getSearchDataProviders(**kwargs)
        oesU = OeSearchUtils(oesmP, fpTypeList=[tup[0] for tup in fpTypeCuttoffList])
        oeioU = OeIoUtils()
        # This will reload the oe binary cache.
        oeMol = oesmP.getMol("004")
        self.assertGreaterEqual(len(list(oeMol.GetAtoms())), 12)

        # matchOpts = "graph-relaxed"
        matchOpts = "graph-strict"
        missTupL = []
        missedD = {}
        missedFpD = {}
        numMols = min(len(ccIdxD), numMols) if numMols else len(ccIdxD)
        logger.info("Begin substructure search w/ finger print filter on %d molecules", numMols)
        # ----
        startTime = time.time()
        for (
            ii,
            ccId,
        ) in enumerate(list(ccIdxD.keys())[:numMols]):
            ccD = ccIdxD[ccId]
            for buildType in buildTypeList:
                if buildType in ccD:
                    startTime1 = time.time()
                    oeMol = oeioU.descriptorToMol(ccD[buildType], buildType, limitPerceptions=limitPerceptions, messageTag=ccId + ":" + buildType)
                    if not oeMol:
                        logger.debug("%s build failed for %s - skipping", ccId, buildType)
                        continue
                    maxHits = 0
                    minHits = maxFpResults
                    selfHit = False
                    for fpType, minFpScore in fpTypeCuttoffList:
                        retStatus, mL = oesU.searchSubStructureWithFingerPrint(oeMol, fpType, minFpScore, maxFpResults, matchOpts=matchOpts)
                        self.assertTrue(retStatus)
                        logger.debug("%s fpType %r hits %d", ccId, fpType, len(mL))
                        maxHits = max(maxHits, len(mL))
                        minHits = min(minHits, len(mL))
                        matchedSelf = self.__resultContains(ccId, mL)
                        selfHit = selfHit or matchedSelf
                        if not matchedSelf:
                            missedFpD.setdefault(ccId, []).append((buildType, fpType, len(mL)))
                    if not selfHit:
                        missedD.setdefault(ccId, []).append(buildType)

                    if maxHits < 1 or not selfHit:
                        logger.info("%s (%r) buildType %r min hits %d max hits %d (%.4f seconds)", ccId, selfHit, buildType, minHits, maxHits, time.time() - startTime1)
                else:
                    logger.debug("%s missing descriptor %r", ccId, buildType)
            if ii % 100 == 0:
                logger.info("Completed %d of %d missed count %d", ii, numMols, len(missedD))
        #
        for ccId, missL in missedD.items():
            logger.info("%s missed list %r", ccId, missL)
            if ccId in missedFpD:
                logger.info("%s unmatched for fpTypes %r", ccId, missedFpD[ccId])
        # ----
        if doDisplay:
            mD = {}
            for missTup in missTupL:
                mD.setdefault(missTup[0], []).append(missTup[1])

            for ccId, buildTypeL in mD.items():
                idxD = ccIdxD[ccId]
                if "oe-iso-smiles" in idxD:
                    for buildType in buildTypeL:
                        self.__displayAlignedDescriptorPair(ccId, idxD["oe-iso-smiles"], "oe-iso-smiles", idxD[buildType], buildType, title=None, limitPerceptions=True)

        logger.info("%s fingerprints search on %d in (%.4f seconds)", len(fpTypeCuttoffList), numMols, time.time() - startTime)
        return True

    def testSubStructureSearchScreenedAbbrev(self):
        """Screened substructure search."""
        myKwargs = {
            "ccUrlTarget": self.__ccUrlTarget,
            "birdUrlTarget": self.__birdUrlTarget,
            "cachePath": self.__cachePath,
            "useCache": True,
            "ccFileNamePrefix": "cc-abbrev",
            "oeFileNamePrefix": "oe-abbrev",
            "limitPerceptions": False,
            "buildTypeList": ["oe-iso-smiles", "oe-smiles", "cactvs-iso-smiles", "cactvs-smiles"],
            "screenTypeList": ["SMARTS"],
        }
        return self.__subStructureSearchScreened(self.__numMols, **myKwargs)

    @unittest.skipIf(skipFlag, "Long troubleshooting test")
    def testSubStructureSearchScreenedFiltered(self):
        """Screened substructure search."""
        numMols = 5000
        myKwargs = {
            "cachePath": self.__cachePath,
            "useCache": True,
            "ccFileNamePrefix": "cc-filtered",
            "oeFileNamePrefix": "oe-filtered",
            "limitPerceptions": False,
            "buildTypeList": ["oe-iso-smiles", "oe-smiles", "cactvs-iso-smiles", "cactvs-smiles"],
            "screenTypeList": ["SMARTS"],
        }
        return self.__subStructureSearchScreened(numMols, **myKwargs)

    def __subStructureSearchScreened(self, numMols, **kwargs):
        #
        buildTypeList = kwargs.get("buildTypeList", ["oe-iso-smiles"])
        screenTypeList = kwargs.get("screenTypeList", ["SMARTS"])
        oesmP, ccIdxD = self.__getSearchDataProviders(**kwargs)
        for screenType in screenTypeList:
            oesU = OeSearchUtils(oesmP, screenType=screenType, numProc=self.__numProc)
            oeioU = OeIoUtils()
            #
            missL = []
            numMols = min(len(ccIdxD), numMols) if numMols else len(ccIdxD)
            for (
                ii,
                ccId,
            ) in enumerate(list(ccIdxD.keys())[:numMols]):
                ccD = ccIdxD[ccId]
                for buildType in buildTypeList:
                    if buildType in ccD:
                        if screenType == "SMARTS":
                            smiles = oeioU.descriptorToSmiles(ccD[buildType], buildType, messageTag=ccId + ":" + buildType)
                            oeQMol = oeioU.descriptorToMol(smiles, "SMARTS", messageTag=ccId + ":" + buildType)
                        else:
                            oeQMol = oeioU.descriptorToQMol(ccD[buildType], "SMARTS", messageTag=ccId + ":" + buildType)
                        if not oeQMol:
                            logger.debug("%s build failed for %s - skipping", ccId, buildType)
                            continue
                        # ----
                        startTime = time.time()
                        retStatus, mL = oesU.searchSubStructureScreened(oeQMol, maxMatches=100)
                        if retStatus:
                            logger.debug("%s - %s - %s (status=%r) match length %d in (%.4f seconds)", ccId, buildType, screenType, retStatus, len(mL), time.time() - startTime)
                        if not self.__resultContains(ccId, mL):
                            missL.append((ccId, buildType, screenType))
                        # ----
                if ii % 100 == 0:
                    logger.info("Completed %d of %d missed count %d", ii, numMols, len(missL))
            logger.info("Screen %r missed searches (%d) %r", screenType, len(missL), missL)
        return True

    def __displayAlignedDescriptorPair(self, ccId, descrRef, buildTypeRef, descrFit, buildTypeFit, title=None, limitPerceptions=True):
        oeioU = OeIoUtils()
        oeMolRef = oeioU.descriptorToMol(descrRef, buildTypeRef, limitPerceptions=limitPerceptions, messageTag=ccId + ":" + buildTypeRef)
        oeMolFit = oeioU.descriptorToMol(descrFit, buildTypeFit, limitPerceptions=limitPerceptions, messageTag=ccId + ":" + buildTypeFit)
        #
        oed = OeDepictMCSAlignPage()
        oed.setSearchType(sType="graph-relaxed", minAtomMatchFraction=0.50)
        oed.setDisplayOptions(
            labelAtomName=True, labelAtomCIPStereo=True, labelAtomIndex=False, labelBondIndex=False, highlightStyleFit="ballAndStickInverse", bondDisplayWidth=0.5
        )
        oed.setRefMol(oeMolRef, ccId)
        oed.setFitMol(oeMolFit, ccId)
        myTitle = title if title else buildTypeRef + "-" + buildTypeFit
        imgPath = os.path.join(self.__workPath, myTitle + "-" + ccId + ".svg")
        logger.info("Using image path %r", imgPath)
        aML = oed.alignPair(imagePath=imgPath)
        if aML:
            logger.info("%s aligned image path %r", ccId, imgPath)
            for (rCC, rAt, tCC, tAt) in aML:
                logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)


def subStructureSearch():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(OeSearchIndexUtilsTests("testSubStructureSearchExhaustiveAbbrev"))
    suiteSelect.addTest(OeSearchIndexUtilsTests("testSubStructureSearchWithFpAbbrev"))
    suiteSelect.addTest(OeSearchIndexUtilsTests("testSubStructureSearchScreenedAbbrev"))
    # suiteSelect.addTest(OeSearchIndexUtilsTests("testSubStructureSearchScreenedFiltered"))
    return suiteSelect


def fingerprintSearch():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(OeSearchIndexUtilsTests("testFingerPrintSearchAbbrev"))
    suiteSelect.addTest(OeSearchIndexUtilsTests("testFingerPrintScoresAbbrev"))
    return suiteSelect


def fullSearchTests():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(OeSearchIndexUtilsTests("testSubStructureSearchWithFpFull"))
    suiteSelect.addTest(OeSearchIndexUtilsTests("testFingerPrintScoresFull"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = fullSearchTests()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    OeOeIoUtilsTests.py
# Author:  jdw
# Date:    28-Oct-2019
# Version: 0.001
#
# Updates:
#
##
"""
A collection of tests for OeIoUtils() operations.

"""
import logging
import os
import unittest

from rcsb.utils.chem.ChemCompMoleculeProvider import ChemCompMoleculeProvider
from rcsb.utils.chem.OeIoUtils import OeIoUtils
from rcsb.utils.chem.OeMoleculeFactory import OeMoleculeFactory
from rcsb.utils.io.MarshalUtil import MarshalUtil

# from rcsb.utils.chem.PdbxChemComp import PdbxChemCompIt

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class OeIoUtilsTests(unittest.TestCase):
    skipFlag = True

    def setUp(self):
        #
        self.__workPath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__molfileDirPath = os.path.join(self.__cachePath, "molfiles")
        self.__ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif")
        self.__birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif")
        self.__molLimit = None

    def tearDown(self):
        pass

    def __getChemCompDefs(self, molLimit=None):
        ccMolD = {}
        try:
            useCache = True
            ccFileNamePrefix = "cc-abbrev"
            ccmP = ChemCompMoleculeProvider(
                ccUrlTarget=self.__ccUrlTarget,
                birdUrlTarget=self.__birdUrlTarget,
                cachePath=self.__cachePath,
                useCache=useCache,
                ccFileNamePrefix=ccFileNamePrefix,
                molLimit=molLimit,
            )
            ok = ccmP.testCache(minCount=molLimit)
            self.assertTrue(ok)
            ccMolD = ccmP.getMolD()
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ccMolD

    def testIoOps(self):
        """Test IO operation on generated related molecules"""
        try:
            oeIoU = OeIoUtils()
            mU = MarshalUtil()
            mU.mkdir(self.__molfileDirPath)
            ccMolD = self.__getChemCompDefs()
            oemf = OeMoleculeFactory()
            for ccId, ccObj in list(ccMolD.items())[:10]:
                # ----
                tId = oemf.setChemCompDef(ccObj)
                self.assertEqual(tId, ccId)
                relatedIdxD = oemf.buildRelated(limitPerceptions=False)
                logger.info("%s generated %d molecular forms", ccId, len(relatedIdxD))
                for sId, idxD in relatedIdxD.items():
                    logger.info("sId %r smiles %r", sId, idxD["smiles"])
                    mol2Path = os.path.join(self.__molfileDirPath, sId + ".mol2")
                    oeMol = oeIoU.descriptorToMol(idxD["smiles"], "oe-iso-smiles", limitPerceptions=False, messageTag=None)
                    oeIoU.write(mol2Path, oeMol, constantMol=True, addSdTags=True)
                    sdfPath = os.path.join(self.__molfileDirPath, sId + ".mol")
                    oeMol = oeIoU.descriptorToMol(idxD["smiles"], "oe-iso-smiles", limitPerceptions=False, messageTag=None)
                    oeIoU.write(sdfPath, oeMol, constantMol=True, addSdTags=True)
                # ----
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testRoundTripOps(self):
        """Test IO operation on generated related molecules"""
        try:
            oeIoU = OeIoUtils()
            mU = MarshalUtil()
            mU.mkdir(self.__molfileDirPath)
            ccMolD = self.__getChemCompDefs()
            oemf = OeMoleculeFactory()
            for ccId, ccObj in list(ccMolD.items())[:10]:
                # ----
                tId = oemf.setChemCompDef(ccObj)
                self.assertEqual(tId, ccId)
                relatedIdxD = oemf.buildRelated(limitPerceptions=False)
                logger.info("%s generated %d molecular forms", ccId, len(relatedIdxD))
                for sId, idxD in relatedIdxD.items():
                    logger.info("sId %r smiles %r", sId, idxD["smiles"])
                    mol2Path = os.path.join(self.__molfileDirPath, sId + ".mol2")
                    oeMol = oeIoU.descriptorToMol(idxD["smiles"], "oe-iso-smiles", limitPerceptions=False, messageTag=None)
                    oeIoU.write(mol2Path, oeMol, constantMol=True, addSdTags=True)
                    tMolL = oeIoU.fileToMols(mol2Path)
                    #
                    nextMol2Path = os.path.join(self.__molfileDirPath, sId + "-next.mol2")
                    oeIoU.write(nextMol2Path, tMolL[0], constantMol=True, addSdTags=True)

                    sdfPath = os.path.join(self.__molfileDirPath, sId + ".mol")
                    oeMol = oeIoU.descriptorToMol(idxD["smiles"], "oe-iso-smiles", limitPerceptions=False, messageTag=None)
                    oeIoU.write(sdfPath, oeMol, constantMol=True, addSdTags=True)
                    #
                    tMolL = oeIoU.fileToMols(sdfPath)
                    nextSdfPath = os.path.join(self.__molfileDirPath, sId + "-next.sdf")
                    oeIoU.write(nextSdfPath, tMolL[0], constantMol=True, addSdTags=True)
                # ----
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def suiteIoOpsTests():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(OeIoUtilsTests("testIoOps"))
    return suiteSelect


if __name__ == "__main__":
    #
    mySuite1 = suiteIoOpsTests()
    unittest.TextTestRunner(verbosity=2).run(mySuite1)
    #



##
# File:    OeChemCompTests.py
# Author:  jdw
# Date:    28-Oct-2019
# Version: 0.001
#
# Updates:
#
##
"""
A collection of tests of OeChemCompUtils.

"""
import logging
import os
import unittest

from rcsb.utils.chem.ChemCompMoleculeProvider import ChemCompMoleculeProvider
from rcsb.utils.chem.OeChemCompUtils import OeChemCompUtils
from rcsb.utils.chem.OeMoleculeFactory import OeMoleculeFactory

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class OeChemCompTests(unittest.TestCase):
    skipFlag = True

    def setUp(self):
        #
        self.__workPath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__ccCifPath = os.path.join(self.__cachePath, "cc-cif")
        self.__ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif")
        self.__birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif")
        self.__molLimit = None

    def tearDown(self):
        pass

    def __getChemCompDefs(self, molLimit=None):
        ccMolD = {}
        try:
            useCache = True
            ccFileNamePrefix = "cc-abbrev"
            ccmP = ChemCompMoleculeProvider(
                ccUrlTarget=self.__ccUrlTarget,
                birdUrlTarget=self.__birdUrlTarget,
                cachePath=self.__cachePath,
                useCache=useCache,
                ccFileNamePrefix=ccFileNamePrefix,
                molLimit=molLimit,
            )
            ok = ccmP.testCache(minCount=molLimit)
            self.assertTrue(ok)
            ccMolD = ccmP.getMolD()
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ccMolD

    def testBuildCifFromOE(self):
        """Build chemical component definitions from OE Mol object"""
        try:
            ccMolD = self.__getChemCompDefs()
            oemf = OeMoleculeFactory()
            #
            for ccId, ccObj in list(ccMolD.items())[:10]:
                # ----
                tId = oemf.setChemCompDef(ccObj)
                self.assertEqual(tId, ccId)
                ok = oemf.build(molBuildType="model-xyz")
                self.assertTrue(ok)
                fp = os.path.join(self.__ccCifPath, ccId + "-gen.cif")
                oeMol = oemf.getMol()
                oeccU = OeChemCompUtils()
                ok = oeccU.addOeMol(ccId, oeMol, missingModelXyz=False, writeIdealXyz=False)
                self.assertTrue(ok)
                ok = oeccU.write(fp)

                # ----
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def suiteBuildCifTests():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(OeChemCompTests("testBuildCifFromOE"))
    return suiteSelect


if __name__ == "__main__":
    #
    mySuite1 = suiteBuildCifTests()
    unittest.TextTestRunner(verbosity=2).run(mySuite1)
    #



##
# File:    ChemCompSearchIndexProviderTests.py
# Author:  J. Westbrook
# Date:    3-Mar-2020
# Version: 0.001
#
# Update:
#
#
##
"""
Tests for utilities to generate a search index for reasonable tautomer and protomer
configurations for core PDB chemical component definitions.

"""
import logging
import os
import platform
import resource
import time
import unittest

from importlib.metadata import version as get_package_version
from rcsb.utils.chem.ChemCompSearchIndexProvider import ChemCompSearchIndexProvider

__version__ = get_package_version("rcsb.utils.chem")

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class ChemCompSearchIndexProviderTests(unittest.TestCase):
    skipFlag = True

    def setUp(self):
        self.__startTime = time.time()
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif")
        self.__birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif")
        #
        logger.debug("Running tests on version %s", __version__)
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testChemCompSearchIndexCacheFilesAbbrev(self):
        """Test search index constructure for an abbreviated chemical component resource file."""
        self.__testBuildSearchIndexCacheFiles(ccUrlTarget=self.__ccUrlTarget, birdUrlTarget=self.__birdUrlTarget, ccFileNamePrefix="cc-abbrev")

    def testChemCompSearchIndexCacheFilesAbbrevMp(self):
        """Test search index constructure for an abbreviated chemical component resource file (multi).
        Creates index length - 2837  411s on numproc 4 on macbookpro .383 GB resident mem
        """
        self.__testBuildSearchIndexCacheFiles(ccUrlTarget=self.__ccUrlTarget, birdUrlTarget=self.__birdUrlTarget, ccFileNamePrefix="cc-abbrev", numProc=4)

    @unittest.skipIf(skipFlag, "Long test")
    def testChemCompSearchIndexCacheFilesFullMp(self):
        """Test search index construction of full chemical component resource files.

        461 s (numproc=12 json format) 7.1 GB resident mem linux len=121079
        """
        self.__testBuildSearchIndexCacheFiles(ccFileNamePrefix="cc-full", numProc=4)

    @unittest.skipIf(skipFlag, "Long test")
    def testChemCompSearchIndexCacheFilesFiltered(self):
        """Test search index construction of a filtered subset of chemical component definitions."""
        self.__testBuildSearchIndexCacheFiles(ccFileNamePrefix="cc-filtered")

    def __testBuildSearchIndexCacheFiles(self, **kwargs):
        """Test build search index chemical component cache files from the input component dictionaries"""
        molLimit = kwargs.get("molLimit", None)
        useCache = kwargs.get("useCache", False)
        logSizes = kwargs.get("logSizes", False)
        limitPerceptions = kwargs.get("limitPerceptions", False)
        numProc = kwargs.get("numProc", 1)
        maxChunkSize = kwargs.get("maxChunkSize", 5)
        molLimit = kwargs.get("molLimit", None)
        ccFileNamePrefix = kwargs.get("ccFileNamePrefix", "cc")
        quietFlag = kwargs.get("quietFlag", True)
        ccUrlTarget = kwargs.get("ccUrlTarget", None)
        birdUrlTarget = kwargs.get("birdUrlTarget", None)
        #
        ccsiP = ChemCompSearchIndexProvider(
            ccUrlTarget=ccUrlTarget,
            birdUrlTarget=birdUrlTarget,
            cachePath=self.__cachePath,
            useCache=useCache,
            molLimit=molLimit,
            ccFileNamePrefix=ccFileNamePrefix,
            limitPerceptions=limitPerceptions,
            numProc=numProc,
            maxChunkSize=maxChunkSize,
            quietFlag=quietFlag,
        )
        ok = ccsiP.testCache(minCount=molLimit, logSizes=logSizes)
        self.assertTrue(ok)
        logger.info(" ******* Completed operation ******** ")
        #
        return ccsiP

    def testFormulaMatch(self):
        """Test formula match   ..."""
        ccsidxP = self.__testBuildSearchIndexCacheFiles(
            ccUrlTarget=self.__ccUrlTarget, birdUrlTarget=self.__birdUrlTarget, logSizes=False, useCache=True, ccFileNamePrefix="cc-abbrev"
        )
        ccidxD = ccsidxP.getIndex()
        logger.info("Matching formula for %d definitions", len(ccidxD))
        for ccId, idxD in ccidxD.items():
            startTime = time.time()
            fQueryD = {el: {"min": eCount, "max": eCount} for el, eCount in idxD["type-counts"].items()}
            if fQueryD:
                rL = ccsidxP.matchMolecularFormulaRange(fQueryD, matchSubset=False)
                logger.debug("%s formula matches %r (%.4f seconds)", ccId, rL, time.time() - startTime)
                ok = self.__resultContains(ccId, rL)
                if not ok:
                    logger.info("%s formula not matched %r %r  (%.4f seconds)", ccId, rL, fQueryD, time.time() - startTime)
                self.assertTrue(ok)

    def testFormulaSubsetMatchAbbrev(self):
        """Test formula range match on abbreviated data set  ..."""
        ccsidxP = self.__testBuildSearchIndexCacheFiles(
            ccUrlTarget=self.__ccUrlTarget, birdUrlTarget=self.__birdUrlTarget, logSizes=False, useCache=True, ccFileNamePrefix="cc-abbrev"
        )
        startTime = time.time()
        fQueryD = {"C": {"min": 50, "max": 65}}
        rL = ccsidxP.matchMolecularFormulaRange(fQueryD, matchSubset=True)
        logger.info("C formula subset matches (%d) (%.4f seconds)", len(rL), time.time() - startTime)
        self.assertGreaterEqual(len(rL), 10)

    @unittest.skipIf(skipFlag, "Long test")
    def testFormulaSubsetMatchFull(self):
        """Test formula range match on the full index   ...
        Full subset search
        """
        ccsidxP = self.__testBuildSearchIndexCacheFiles(useCache=True, numProc=4, ccFileNamePrefix="cc-full")
        startTime = time.time()
        fQueryD = {"C": {"min": 10, "max": 65}, "O": {"min": 1, "max": 20}, "N": {"min": 2, "max": 12}}
        rL = ccsidxP.matchMolecularFormulaRange(fQueryD, matchSubset=True)
        logger.info("Formula subset matches (%d) (%.4f seconds)", len(rL), time.time() - startTime)
        self.assertGreaterEqual(len(rL), 10)

    @unittest.skipIf(skipFlag, "Long test")
    def testMinumumFormulaSubsetFull(self):
        """Test formula filter in minumum formula composition on the full index   ..."""
        ccsidxP = self.__testBuildSearchIndexCacheFiles(useCache=True, numProc=4, ccFileNamePrefix="cc-full")
        startTime = time.time()
        fQueryD = {"C": {"min": 10}, "O": {"min": 1}, "N": {"min": 2}}
        rL = ccsidxP.filterMinimumMolecularFormula(fQueryD)
        logger.info("Formula minimum filters (%d) (%.4f seconds)", len(rL), time.time() - startTime)
        self.assertGreaterEqual(len(rL), 10)

    def __resultContains(self, ccId, matchResultList):
        for matchResult in matchResultList:
            if ccId in matchResult.ccId:
                return True
        return False


def buildCacheFiles():
    suiteSelect = unittest.TestSuite()
    # suiteSelect.addTest(ChemCompSearchIndexProviderTests("testChemCompSearchIndexCacheFilesFiltered"))
    suiteSelect.addTest(ChemCompSearchIndexProviderTests("testChemCompSearchIndexCacheFilesAbbrev"))
    suiteSelect.addTest(ChemCompSearchIndexProviderTests("testChemCompSearchIndexCacheFilesFullMp"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = buildCacheFiles()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    ChemCompMoleculeProvider.py
# Author:  J. Westbrook
# Date:    16-Feb-2020
#
# Updates:
#  26-Aug-2021 jdw add file for obsolete component definitions
##
"""
Utilities to read and serialize the dictionary of PDBx/mmCIF chemical component definitions.
"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"

import logging
import os
import time

from rcsb.utils.chem.PdbxChemComp import PdbxChemCompIt
from rcsb.utils.io.FileUtil import FileUtil
from rcsb.utils.io.IoUtil import getObjSize
from rcsb.utils.io.MarshalUtil import MarshalUtil

# from rcsb.utils.io.SingletonClass import SingletonClass

logger = logging.getLogger(__name__)


class ChemCompMoleculeProvider(object):
    """Utilities to read and serialize the dictionary of PDBx/mmCIF chemical component definitions."""

    def __init__(self, **kwargs):
        # Default source target locators
        self.__ccUrlTarget = kwargs.get("ccUrlTarget", None)
        self.__ccUrlTarget = self.__ccUrlTarget if self.__ccUrlTarget else "http://files.wwpdb.org/pub/pdb/data/monomers/components.cif.gz"
        self.__birdUrlTarget = kwargs.get("birdUrlTarget", None)
        self.__birdUrlTarget = self.__birdUrlTarget if self.__birdUrlTarget else "http://files.wwpdb.org/pub/pdb/data/bird/prd/prdcc-all.cif.gz"
        #
        ccFileNamePrefix = kwargs.get("ccFileNamePrefix", "cc")
        cachePath = kwargs.get("cachePath", ".")
        dirPath = os.path.join(cachePath, "chem_comp")
        useCache = kwargs.get("useCache", True)
        molLimit = kwargs.get("molLimit", 0)
        skipObsolete = kwargs.get("skipObsolete", True)
        # Optional id dictionary filter
        filterIdD = kwargs.get("filterIdD", None)
        #
        self.__mU = MarshalUtil(workPath=dirPath)
        self.__ccMolD = self.__reload(
            self.__ccUrlTarget, self.__birdUrlTarget, ccFileNamePrefix, dirPath, useCache=useCache, molLimit=molLimit, filterIdD=filterIdD, skipObsolete=skipObsolete
        )

    def testCache(self, minCount=None, logSizes=False):
        if logSizes and self.__ccMolD:
            logger.info("ccMolD object size %.2f MB", getObjSize(self.__ccMolD) / 1000000.0)
        ok = self.__ccMolD and len(self.__ccMolD) >= minCount if minCount else self.__ccMolD is not None
        return ok

    def getMolD(self):
        return self.__ccMolD

    def getMol(self, ccId):
        try:
            return self.__ccMolD[ccId]
        except Exception as e:
            logger.debug("Get molecule %r failing with %s", ccId, str(e))
        return None

    def getReleaseStatus(self, ccId):
        try:
            ccIt = iter(PdbxChemCompIt(self.__ccMolD[ccId]))
            ccIt = next(ccIt, None)
            return ccIt.getReleaseStatus() if ccIt else None
        except Exception as e:
            logger.exception("Failing for ccId %r with %s", ccId, str(e))
        return None

    def __reload(self, ccUrlTarget, birdUrlTarget, ccFileNamePrefix, dirPath, useCache=False, molLimit=None, filterIdD=None, skipObsolete=True):
        """Reload or create serialized data dictionary of chemical components.

        Args:
            ccUrlTarget (str): target url for chemical component dictionary resource file
            birdUrlTarget (str): target url for bird dictionary resource file (cc format)
            dirPath (str): path to the directory containing cache files
            useCache (bool):
            molLimit (int): maximum number of definitions to process
            filterIdD (dict): dictionary of selected chemical component identifier codes
            skipObsolete (bool): skip obsolete definitions

         Returns:
            (list): chemical component data containers
        """
        #
        startTime = time.time()
        # This is the naming standard for serialized PDBx/mmCIF component data
        ccDataFilePath = os.path.join(dirPath, "%s-chemical-component-data.pic" % ccFileNamePrefix)
        _, fExt = os.path.splitext(ccDataFilePath)
        ccDataFormat = "json" if fExt == ".json" else "pickle"
        #
        if useCache and self.__mU.exists(ccDataFilePath):
            rdCcObjD = self.__mU.doImport(ccDataFilePath, fmt=ccDataFormat)
            ccObjD = {k: rdCcObjD[k] for k in sorted(rdCcObjD.keys())[:molLimit]} if molLimit else rdCcObjD
            if skipObsolete:
                tD = {}
                for ccId in ccObjD:
                    ccIt = iter(PdbxChemCompIt(ccObjD[ccId]))
                    ccIt = next(ccIt, None)
                    if ccIt.getReleaseStatus() not in ["REL", "REF_ONLY"]:
                        continue
                    tD[ccId] = ccObjD[ccId]
                ccObjD = tD

        else:
            # Source component data files ...
            ccdFilePath = self.__fetchUrl(ccUrlTarget, dirPath, useCache=useCache)
            birdFilePath = self.__fetchUrl(birdUrlTarget, dirPath, useCache=useCache)
            rdCcObjD = self.__readComponentDefinitions(ccdFilePath, birdFilePath, molLimit=molLimit, skipObsolete=skipObsolete)
            ccObjD = {ccId: ccObj for ccId, ccObj in rdCcObjD.items() if ccId in filterIdD} if filterIdD else rdCcObjD
            ok = self.__mU.doExport(ccDataFilePath, ccObjD, fmt=ccDataFormat)
            logger.info("Storing %d definitions (status=%r) path: %s ", len(ccObjD), ok, ccDataFilePath)
        #
        endTime = time.time()
        logger.info("Loaded/reloaded %d definitions (%.4f seconds)", len(ccObjD), endTime - startTime)
        return ccObjD

    def __fetchUrl(self, urlTarget, dirPath, useCache=False):
        fU = FileUtil()
        fn = fU.getFileName(urlTarget)
        filePath = os.path.join(dirPath, fn)
        if not (useCache and fU.exists(filePath)):
            startTime = time.time()
            ok2 = fU.get(urlTarget, filePath)
            endTime = time.time()
            if ok2:
                logger.info("Fetched %s for resource file %s (status = %r) (%.4f seconds)", urlTarget, filePath, ok2, endTime - startTime)
            else:
                logger.error("Failing fetch for %s for resource file %s (status = %r) (%.4f seconds)", urlTarget, filePath, ok2, endTime - startTime)
        #
        return filePath

    def __readComponentDefinitions(self, ccdFilePath, birdFilePath=None, molLimit=None, skipObsolete=True):
        ccObjD = {}
        try:
            startTime = time.time()
            logger.info("Reading definitions from %s", ccdFilePath)
            rdCcObjL = self.__mU.doImport(ccdFilePath, fmt="mmcif")
            endTime = time.time()
            logger.info("Read %s with %d CCD definitions (%.4f seconds)", ccdFilePath, len(rdCcObjL), endTime - startTime)
            # -------
            if birdFilePath:
                startTime = time.time()
                logger.info("Reading definitions from %s", birdFilePath)
                birdCcObjL = self.__mU.doImport(birdFilePath, fmt="mmcif")
                endTime = time.time()
                logger.info("Read %s with %d BIRD definitions (%.4f seconds)", birdFilePath, len(birdCcObjL), endTime - startTime)
                rdCcObjL.extend(birdCcObjL)
            #
            startTime = time.time()
            ccObjL = rdCcObjL[:molLimit] if molLimit else rdCcObjL
            for ccObj in ccObjL:
                ccIt = iter(PdbxChemCompIt(ccObj))
                ccIt = next(ccIt, None)
                ccId = ccIt.getId() if ccIt else ccObj.getName()
                if skipObsolete and ccIt.getReleaseStatus() not in ["REL", "REF_ONLY"]:
                    continue
                ccObjD[ccId] = ccObj
            endTime = time.time()
            logger.info("Processed %d definitions (%.4f seconds)", len(ccObjD), endTime - startTime)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        #
        return ccObjD



##
# File:    ChemCompIndexProviderTests.py
# Author:  J. Westbrook
# Date:    17-Feb-2020
# Version: 0.001
#
# Update:
#
#
##
"""
Tests for utilities to read and process an index of PDB chemical component definitions.

"""
import logging
import os
import platform
import resource
import time
import unittest

from importlib.metadata import version as get_package_version
from rcsb.utils.chem.ChemCompIndexProvider import ChemCompIndexProvider
from rcsb.utils.chem.MolecularFormula import MolecularFormula

__version__ = get_package_version("rcsb.utils.chem")

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class ChemCompIndexProviderTests(unittest.TestCase):
    skipFlag = True

    def setUp(self):
        self.__startTime = time.time()
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif")
        self.__birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif")
        #
        logger.debug("Running tests on version %s", __version__)
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testChemCompIndexCacheFilesAbbrev(self):
        """Test construction of full chemical component resource files."""
        self.__testBuildMoleculeCacheFiles(ccUrlTarget=self.__ccUrlTarget, birdUrlTarget=self.__birdUrlTarget, logSizes=False, useCache=False, ccFileNamePrefix="cc-abbrev")

    @unittest.skipIf(skipFlag, "Long test")
    def testChemCompIndexCacheFilesFull(self):
        """Test construction of full chemical component resource files."""
        self.__testBuildMoleculeCacheFiles(useCache=False, ccFileNamePrefix="cc-full")

    @unittest.skipIf(skipFlag, "Long test")
    def testChemCompIndexCacheFilesFiltered(self):
        """Test construction of a filtered subset of chemical component definitions."""
        self.__testBuildMoleculeCacheFiles(useCache=False, ccFileNamePrefix="cc-filtered")

    def __testBuildMoleculeCacheFiles(self, **kwargs):
        """Test build chemical component cache files from the input component dictionaries"""
        molLimit = kwargs.get("molLimit", None)
        useCache = kwargs.get("useCache", True)
        logSizes = kwargs.get("logSizes", False)
        ccFileNamePrefix = kwargs.get("ccFileNamePrefix", "cc")
        ccUrlTarget = kwargs.get("ccUrlTarget", None)
        birdUrlTarget = kwargs.get("birdUrlTarget", None)
        #
        ccidxP = ChemCompIndexProvider(
            ccUrlTarget=ccUrlTarget, birdUrlTarget=birdUrlTarget, cachePath=self.__cachePath, useCache=useCache, molLimit=molLimit, ccFileNamePrefix=ccFileNamePrefix
        )
        ok = ccidxP.testCache(minCount=molLimit, logSizes=logSizes)
        self.assertTrue(ok)
        logger.info(" ******* Completed operation ******** ")
        #
        return ccidxP

    def testFormulaMatch(self):
        """Test formula match   ..."""
        ccidxP = self.__testBuildMoleculeCacheFiles(ccUrlTarget=self.__ccUrlTarget, birdUrlTarget=self.__birdUrlTarget, logSizes=False, useCache=True, ccFileNamePrefix="cc-abbrev")
        ccidxD = ccidxP.getIndex()
        logger.info("Matching formula for %d definitions", len(ccidxD))
        for ccId, idxD in ccidxD.items():
            startTime = time.time()
            fQueryD = {el: {"min": eCount, "max": eCount} for el, eCount in idxD["type-counts"].items()}
            if fQueryD:
                rL = ccidxP.matchMolecularFormulaRange(fQueryD, matchSubset=False)
                logger.debug("%s formula matches %r (%.4f seconds)", ccId, rL, time.time() - startTime)
                ok = self.__resultContains(ccId, rL)
                if not ok:
                    logger.info("%s formula not matched %r %r  (%.4f seconds)", ccId, rL, fQueryD, time.time() - startTime)
                self.assertTrue(ok)

    def testFormulaSubsetMatch(self):
        """Test formula range match   ..."""
        ccidxP = self.__testBuildMoleculeCacheFiles(ccUrlTarget=self.__ccUrlTarget, birdUrlTarget=self.__birdUrlTarget, logSizes=False, useCache=True, ccFileNamePrefix="cc-abbrev")
        startTime = time.time()
        fQueryD = {"C": {"min": 50, "max": 65}}
        rL = ccidxP.matchMolecularFormulaRange(fQueryD, matchSubset=True)
        logger.info("C formula subset matches (%d) (%.4f seconds)", len(rL), time.time() - startTime)
        self.assertGreaterEqual(len(rL), 10)

    def __resultContains(self, ccId, matchResultList):
        for matchResult in matchResultList:
            if ccId in matchResult.ccId:
                return True
        return False

    def testFormulaStringMatch(self):
        """Test formula match   ..."""
        mf = MolecularFormula()
        ccidxP = self.__testBuildMoleculeCacheFiles(ccUrlTarget=self.__ccUrlTarget, birdUrlTarget=self.__birdUrlTarget, logSizes=False, useCache=True, ccFileNamePrefix="cc-abbrev")
        ccidxD = ccidxP.getIndex()
        logger.info("Matching formula for %d definitions", len(ccidxD))
        for ccId, idxD in ccidxD.items():
            startTime = time.time()
            formula = idxD["formula"]
            eD = mf.parseFormula(formula)
            eDU = {k.upper(): v for k, v in eD.items()}
            eDS = set(eDU.items())
            typeCounts = idxD["type-counts"]
            tCS = set(typeCounts.items())
            ok = eDS == tCS
            if not ok:
                logger.info("%s parsed eD %r typeCounts %r  (%.4f seconds)", ccId, eDS, tCS, time.time() - startTime)
            self.assertTrue(ok)
            # logger.info("%s parsed eD %r typeCounts %r  (%.4f seconds)", ccId, eD, typeCounts, time.time() - startTime)

    def testFormulaParser(self):
        """Test formula parser ..."""
        mf = MolecularFormula()
        for fS in ["co", "CO", "NI", "NiI", "Co", "C1Co1"]:
            eD = mf.parseFormula(fS)
            logger.info("Formula %s %r", fS, eD)


def buildCacheFiles():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ChemCompIndexProviderTests("testChemCompIndexCacheFilesFull"))
    suiteSelect.addTest(ChemCompIndexProviderTests("testChemCompIndexCacheFilesFiltered"))
    suiteSelect.addTest(ChemCompIndexProviderTests("testChemCompIndexCacheFilesAbbrev"))
    return suiteSelect


def matchFormula():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ChemCompIndexProviderTests("testFormulaMatch"))
    suiteSelect.addTest(ChemCompIndexProviderTests("testFormulaSubsetMatch"))
    suiteSelect.addTest(ChemCompIndexProviderTests("testFormulaStringMatch"))
    return suiteSelect


if __name__ == "__main__":

    mySuite = buildCacheFiles()
    unittest.TextTestRunner(verbosity=2).run(mySuite)

    mySuite = matchFormula()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    testOeAlignUtils.py
# Author:  jdw
# Date:    17-Dec-2020
# Version: 0.001
#
# Updates:
##
"""
A collection of tests for SS and MCSS comparison operations.
"""
import logging
import os
import platform
import resource
import time
import unittest

from importlib.metadata import version as get_package_version
from rcsb.utils.chem.OeAlignUtils import OeAlignUtils

__version__ = get_package_version("rcsb.utils.chem")

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()
logger.setLevel(logging.INFO)


class OeAlignUtilsTests(unittest.TestCase):
    def setUp(self):
        self.__verbose = True
        #
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__workPath = os.path.join(HERE, "test-output")
        self.__startTime = time.time()
        logger.debug("Running tests on version %s", __version__)
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testMCSSAlignCCPair(self):
        """Test case -  Simple pairwise MCSS alignment   (cif) """
        try:
            refPath = os.path.join(self.__dataPath, "001.cif")
            fitPath = os.path.join(self.__dataPath, "001.cif")
            oed = OeAlignUtils(workPath=self.__workPath, verbose=self.__verbose, timeOut=120.0)
            oed.setSearchType(sType="relaxed")
            oed.setRefPath(refPath)
            logger.info("Ref path %s", refPath)
            oed.setFitPath(fitPath)
            logger.info("Fit path %s", fitPath)
            (nAtomsRef, refFD, nAtomsFit, fitFD, atomMapL, _) = oed.doAlignMcss(minFrac=0.9)
            self.assertEqual(nAtomsRef, nAtomsFit)
            self.assertEqual(nAtomsRef, len(atomMapL))
            self.assertEqual(refFD["InChIKey"], fitFD["InChIKey"])
            if len(atomMapL) > 0:
                for alm in atomMapL:
                    logger.debug("%5s %5s %5s %-5s %5s %5s %5s %-5s", alm.refId, alm.refAtIdx, alm.refAtNo, alm.refAtName, alm.fitId, alm.fitAtIdx, alm.fitAtNo, alm.fitAtName)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testMCSSAlignCCPairFail(self):
        """Test case -  Timeout """
        try:
            refPath = os.path.join(self.__dataPath, "001.cif")
            fitPath = os.path.join(self.__dataPath, "001.cif")
            oed = OeAlignUtils(workPath=self.__workPath, verbose=self.__verbose, timeOut=1.0)
            oed.setSearchType(sType="relaxed")
            oed.setRefPath(refPath)
            logger.info("Ref path %s", refPath)
            oed.setFitPath(fitPath)
            logger.info("Fit path %s", fitPath)
            oed.doAlignMcss(minFrac=0.9)
        except TimeoutError as e:
            logger.info("Caught timeout with %s", str(e))

    def testMCSSAlignMixedPair(self):
        """Test case -  Simple pairwise MCSS alignment   (sdf/cif) """
        try:
            minFrac = 0.90
            for ccId in ["MAN", "001"]:
                refPath = os.path.join(self.__dataPath, "%s.cif" % ccId)
                fitPath = os.path.join(self.__dataPath, "%s.sdf" % ccId)

                oed = OeAlignUtils(workPath=self.__workPath, verbose=self.__verbose)
                oed.setSearchType(sType="relaxed")
                logger.info("Ref path %s", refPath)
                oed.setRefPath(refPath, title=ccId, suppressHydrogens=False, fType="CC", importType="3D")
                #
                logger.info("Fit path %s", fitPath)
                oed.setFitPath(fitPath, title=ccId, suppressHydrogens=False, fType="sdf", importType="2D")
                #
                (nAtomsRef, refFD, nAtomsFit, fitFD, atomMapL, _) = oed.doAlignMcss(minFrac=minFrac, useExhaustive=True)
                self.assertEqual(nAtomsRef, nAtomsFit)
                self.assertGreaterEqual(nAtomsRef, len(atomMapL) * minFrac)
                self.assertEqual(refFD["SMILES"], fitFD["SMILES"])
                if len(atomMapL) > 0:
                    for alm in atomMapL:
                        logger.debug("%5s %5s %5s %-5s %5s %5s %5s %-5s", alm.refId, alm.refAtIdx, alm.refAtNo, alm.refAtName, alm.fitId, alm.fitAtIdx, alm.fitAtNo, alm.fitAtName)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testSSAlignMixedPair2D(self):
        """Test case -  Simple pairwise SS alignment   (sdf/cif) """
        try:
            for ccId in ["MAN", "001"]:
                refPath = os.path.join(self.__dataPath, "%s.cif" % ccId)
                fitPath = os.path.join(self.__dataPath, "%s.sdf" % ccId)

                oed = OeAlignUtils(workPath=self.__workPath, verbose=self.__verbose)
                oed.setSearchType(sType="strict")
                logger.info("Ref path %s", refPath)
                oed.setRefPath(refPath)
                #
                logger.info("Fit path %s", fitPath)
                oed.setFitPath(fitPath, title=None, suppressHydrogens=False, fType="sdf", importType="2D", largestPart=True)
                #
                (nAtomsRef, refFD, nAtomsFit, fitFD, atomMapL, _) = oed.doAlignSs(unique=True, maxMatches=1)
                self.assertEqual(nAtomsRef, nAtomsFit)
                self.assertEqual(nAtomsRef, len(atomMapL))
                self.assertEqual(refFD["SMILES"], fitFD["SMILES"])
                if len(atomMapL) > 0:
                    for alm in atomMapL:
                        logger.debug("%5s %5s %5s %-5s %5s %5s %5s %-5s", alm.refId, alm.refAtIdx, alm.refAtNo, alm.refAtName, alm.fitId, alm.fitAtIdx, alm.fitAtNo, alm.fitAtName)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testSSAlignMixedPair3D(self):
        """Test case -  Simple pairwise SS alignment   (sdf/cif) """
        try:
            for ccId in ["CBO"]:
                refPath = os.path.join(self.__dataPath, "%s.cif" % ccId)
                fitPath = os.path.join(self.__dataPath, "%s.sdf" % ccId)

                oed = OeAlignUtils(workPath=self.__workPath, verbose=self.__verbose)
                oed.setSearchType(sType="strict")
                logger.info("Ref path %s", refPath)
                oed.setRefPath(refPath)
                #
                logger.info("Fit path %s", fitPath)
                oed.setFitPath(fitPath, title=None, suppressHydrogens=False, fType="sdf", importType="3D", largestPart=True)
                #
                (nAtomsRef, refFD, nAtomsFit, fitFD, atomMapL, _) = oed.doAlignSs(unique=True, maxMatches=1)
                self.assertEqual(nAtomsRef, nAtomsFit)
                self.assertEqual(nAtomsRef, len(atomMapL))
                self.assertEqual(refFD["SMILES"], fitFD["SMILES"])
                if len(atomMapL) > 0:
                    for alm in atomMapL:
                        logger.debug("%5s %5s %5s %-5s %5s %5s %5s %-5s", alm.refId, alm.refAtIdx, alm.refAtNo, alm.refAtName, alm.fitId, alm.fitAtIdx, alm.fitAtNo, alm.fitAtName)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def suiteAlignPair():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(OeAlignUtilsTests("testMCSSAlignCCPair"))
    suiteSelect.addTest(OeAlignUtilsTests("testMCSSAlignMixedPair"))
    suiteSelect.addTest(OeAlignUtilsTests("testSSAlignMixedPair"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = suiteAlignPair()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    OeDepictAlignTests.py
# Author:  jdw
# Date:    28-Oct-2019
# Version: 0.001
#
# Updates:
#
##
"""
A collection of tests for the OEDepictAlign and related classes which perform
MCSS comparison and aligned depiction.

"""
import logging
import os
import unittest

from rcsb.utils.chem.OeDepictAlign import OeDepictMCSAlign, OeDepictMCSAlignMultiPage, OeDepictMCSAlignPage, OeDepictSubStructureAlign, OeMCSAlignUtil
from rcsb.utils.chem.OeMoleculeProvider import OeMoleculeProvider

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


class OeDepictAlignTests(unittest.TestCase):
    skipFlag = True

    def setUp(self):
        #
        self.__workPath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif")
        self.__birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif")
        #
        # self.__idList = ["000", "001", "002", "003", "004", "0K3"]
        self.__idList = ["000", "001", "002", "004", "0K3"]
        self.__refId = "000"
        self.__pairIdList = [("000", "000"), ("001", "001"), ("002", "002"), ("002", "003"), ("004", "0K3")]
        #
        # self.__idList = ["cg1", "atp", "gtp", "A", "C", "G", "DG"]
        # self.__pairIdList = [("c", "cg1"), ("c", "atp"), ("c", "gtp"), ("c", "A"), ("c", "C"), ("c", "G"), ("c", "DG")]
        self.__ssPairIdList = self.__pairIdList = [("000", "000"), ("001", "001"), ("002", "002")]
        #
        self.__rnaPairIdList = [
            ("ZAD", "A"),
            ("1MA", "A"),
            ("MA6", "A"),
            ("SRA", "A"),
            ("12A", "A"),
            ("2MA", "A"),
            ("LCA", "A"),
            ("T6A", "A"),
            ("MAD", "A"),
            ("RIA", "A"),
            ("A3P", "A"),
            ("A2L", "A"),
            ("A23", "A"),
            ("8AN", "A"),
            ("PU", "A"),
            ("6IA", "A"),
            ("P5P", "A"),
            ("PPU", "A"),
            ("5FA", "A"),
            ("A2M", "A"),
            ("MTU", "A"),
            ("A44", "A"),
            ("AVC", "A"),
            ("MGQ", "A"),
            ("AP7", "A"),
            ("MIA", "A"),
            ("AET", "A"),
            ("A5O", "A"),
            ("N5M", "C"),
            ("OMC", "C"),
            ("10C", "C"),
            ("ZCY", "C"),
            ("ZBC", "C"),
            ("PMT", "C"),
            ("M5M", "C"),
            ("IC", "C"),
            ("S4C", "C"),
            ("M4C", "C"),
            ("CBV", "C"),
            ("CH", "C"),
            ("CCC", "C"),
            ("CSF", "C"),
            ("C43", "C"),
            ("C31", "C"),
            ("C2L", "C"),
            ("1SC", "C"),
            ("A5M", "C"),
            ("5IC", "C"),
            ("5MC", "C"),
            ("4OC", "C"),
            ("TPG", "G"),
            ("OMG", "G"),
            ("1MG", "G"),
            ("YYG", "G"),
            ("YG", "G"),
            ("XTS", "G"),
            ("PGP", "G"),
            ("23G", "G"),
            ("G2L", "G"),
            ("2MG", "G"),
            ("GRB", "G"),
            ("QUO", "G"),
            ("ZGU", "G"),
            ("M2G", "G"),
            ("CG1", "G"),
            ("KAG", "G"),
            ("MGV", "G"),
            ("IG", "G"),
            ("GTP", "G"),
            ("GOM", "G"),
            ("GH3", "G"),
            ("7MG", "G"),
            ("G7M", "G"),
            ("G46", "G"),
            ("G48", "G"),
            ("GDP", "G"),
            ("G25", "G"),
            ("O2G", "G"),
            ("GAO", "G"),
            ("N6G", "G"),
            ("T41", "T"),
            ("T38", "T"),
            ("T2S", "T"),
            ("T23", "T"),
            ("SMT", "T"),
            ("T39", "T"),
            ("ZTH", "T"),
            ("UAR", "U"),
            ("UR3", "U"),
            ("UMP", "U"),
            ("4SU", "U"),
            ("UD5", "U"),
            ("U8U", "U"),
            ("U37", "U"),
            ("U36", "U"),
            ("URD", "U"),
            ("US5", "U"),
            ("5MU", "U"),
            ("125", "U"),
            ("OMU", "U"),
            ("126", "U"),
            ("ZBU", "U"),
            ("127", "U"),
            ("ONE", "U"),
            ("5BU", "U"),
            ("70U", "U"),
            ("U34", "U"),
            ("U31", "U"),
            ("5FU", "U"),
            ("2OM", "U"),
            ("CNU", "U"),
            ("RSQ", "U"),
            ("RUS", "U"),
            ("SUR", "U"),
            ("SSU", "U"),
            ("2MU", "U"),
            ("3AU", "U"),
            ("PYO", "U"),
            ("2AU", "U"),
            ("U2P", "U"),
            ("U2L", "U"),
            ("IU", "U"),
            ("PSU", "U"),
            ("3MU", "U"),
            ("FHU", "U"),
            ("MNU", "U"),
            ("H2U", "U"),
            ("3TD", "U"),
        ]
        #
        self.__oeMolD = self.__getCache(molBuildType="model-xyz", useCache=True)

    def tearDown(self):
        pass

    def __getCache(self, molBuildType="model-xyz", useCache=True):
        oemp = OeMoleculeProvider(
            ccUrlTarget=self.__ccUrlTarget,
            birdUrlTarget=self.__birdUrlTarget,
            ccFileNamePrefix="cc-abbrev",
            cachePath=self.__cachePath,
            molBuildType=molBuildType,
            useCache=useCache,
        )
        ok = oemp.testCache()
        self.assertTrue(ok)
        return oemp.getOeMolD()

    def __getMolDepictList(self, ccIdList):
        dL = []
        for ccId in ccIdList:
            ccId = ccId.upper()
            if ccId in self.__oeMolD:
                dL.append((self.__oeMolD[ccId], ccId, "Title: " + ccId))
            else:
                logger.info("Missing molecule %r", ccId)
        return dL

    def testMCSAlignPairDepict(self):
        """Test case -  Simple pairwise MCSS alignment  -  Each aligned pair output to a separate image file"""
        try:
            oed = OeDepictMCSAlignPage()
            oed.setDisplayOptions(
                labelAtomName=True, labelAtomCIPStereo=True, labelAtomIndex=False, labelBondIndex=False, highlightStyleFit="ballAndStickInverse", bondDisplayWidth=0.5
            )

            oed.setRefMol(self.__oeMolD[self.__refId], self.__refId)
            fitTupList = self.__getMolDepictList(self.__idList)

            for fitTup in fitTupList:
                oed.setFitMol(fitTup[0], fitTup[1])
                imgPath = os.path.join(self.__workPath, "ref-" + self.__refId + "-trg-" + fitTup[1] + ".svg")
                logger.info("Using image path %r", imgPath)
                aML = oed.alignPair(imagePath=imgPath)
                # self.assertGreater(len(aML), 2)
                if aML:
                    for (rCC, rAt, tCC, tAt) in aML:
                        logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testMCSRelaxAlignPairDepict(self):
        """Test case -  Relaxed pairwise MCSS alignment"""
        try:
            oed = OeDepictMCSAlignPage()
            oed.setSearchType(sType="relaxed")
            # oed.setDisplayOptions(
            #    labelAtomName=True, labelAtomCIPStereo=True, labelAtomIndex=False, labelBondIndex=False, highlightStyleFit="ballAndStickInverse", bondDisplayWidth=0.5
            # )

            oed.setRefMol(self.__oeMolD[self.__refId], self.__refId)
            fitTupList = self.__getMolDepictList(self.__idList)
            for fitTup in fitTupList:
                oed.setFitMol(fitTup[0], fitTup[1])
                fName = os.path.join(self.__workPath, "relaxed-ref-" + self.__refId + "-trg-" + fitTup[1] + ".svg")

                oed.setDisplayOptions(
                    imageSizeX=2000,
                    imageSizeY=1000,
                    labelAtomName=True,
                    labelAtomCIPStereo=True,
                    labelAtomIndex=False,
                    labelBondIndex=False,
                    highlightStyleFit="ballAndStickInverse",
                    bondDisplayWidth=1.0,
                )
                aML = oed.alignPair(imagePath=fName)
                logger.debug("refId %r fitId %r aML %r", self.__refId, fitTup[1], len(aML))
                self.assertGreater(len(aML), 1)
                if aML:
                    for (rCC, rAt, tCC, tAt) in aML:
                        logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testMCSAlignFitListDepictMulti(self):
        """Test case -  List view of pairwise MCS alignment - multipage output"""
        try:
            oed = OeDepictMCSAlignMultiPage()
            oed.setRefMol(self.__oeMolD[self.__refId.upper()], self.__refId)
            fitMolList = [self.__oeMolD[ccId.upper()] for ccId in self.__idList]
            oed.addFitMolList(fitMolList, suppressHydrogens=False, imageDirPath=self.__workPath)
            oed.setDisplayOptions(
                labelAtomName=True,
                labelAtomCIPStereo=True,
                labelAtomIndex=False,
                labelBondIndex=False,
                highlightStyleFit="ballAndStickInverse",
                gridRows=3,
                gridCols=3,
                bondDisplayWidth=0.5,
            )
            imagePath = os.path.join(self.__workPath, "list-example-mcs-alignment.pdf")
            aML = oed.alignOneWithListMulti(imagePath=imagePath)
            self.assertGreater(len(aML), 2)
            if aML:
                for (rCC, rAt, tCC, tAt) in aML:
                    logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testMCSAlignPairListDepictPortrait(self):
        """Test case -  List view MCS alignment using pair id list input"""
        try:
            pairList = [(tup[0].upper(), self.__oeMolD[tup[0].upper()], tup[1].upper(), self.__oeMolD[tup[1].upper()]) for tup in self.__pairIdList]
            oed = OeDepictMCSAlignMultiPage()
            oed.setPairMolList(pairList)

            oed.setDisplayOptions(
                labelAtomName=True,
                labelAtomCIPStereo=True,
                labelAtomIndex=False,
                labelBondIndex=False,
                highlightStyleFit="ballAndStickInverse",
                pageOrientation="portrait",
                gridRows=4,
                bondDisplayWidth=0.5,
            )

            imagePath = os.path.join(self.__workPath, "pair-list-example-mcs-alignment-portrait.pdf")
            aML = oed.alignPairListMulti(imagePath=imagePath)
            self.assertGreater(len(aML), 2)
            if aML:
                for (rCC, rAt, tCC, tAt) in aML:
                    logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    @unittest.skipIf(skipFlag, "Large data dependency")
    def testMCSAlignRnaPairListDepict(self):
        """Test case -  Modified RNA nucleotide alignment with parent nucleotied using pair list input"""
        try:
            pairList = [(tup[0].upper(), self.__oeMolD[tup[0].upper()], tup[1].upper(), self.__oeMolD[tup[1].upper()]) for tup in self.__rnaPairIdList]
            oed = OeDepictMCSAlignMultiPage()
            oed.setPairMolList(pairList)
            oed.setDisplayOptions(
                labelAtomName=True, labelAtomCIPStereo=True, labelAtomIndex=False, labelBondIndex=False, highlightStyleFit="ballAndStickInverse", bondDisplayWidth=0.5
            )

            imagePath = os.path.join(self.__workPath, "rna-modified-pair-alignment.pdf")
            aML = oed.alignPairListMulti(imagePath=imagePath)
            self.assertGreater(len(aML), 2)
            if aML:
                for (rCC, rAt, tCC, tAt) in aML:
                    logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    @unittest.skipIf(skipFlag, "Large data dependency")
    def testMCSAlignAtomMap(self):
        """Test case -  match test with return of atom maps"""
        try:
            pairList = [(tup[0].upper(), self.__oeMolD[tup[0].upper()], tup[1].upper(), self.__oeMolD[tup[1].upper()]) for tup in self.__rnaPairIdList]
            oed = OeMCSAlignUtil()
            for refId, refMol, fitId, fitMol in pairList:
                oed.setRefMol(refMol, refId)
                oed.setFitMol(fitMol, fitId)
                aML = oed.doAlign()
                self.assertGreater(len(aML), 2)
                if aML:
                    logger.debug("Match length %3d for: %s %s", len(aML), refId, fitId)
                else:
                    logger.info("Match failed for: %s %s", refId, fitId)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testMCSAlignListMultiDepict(self):
        """Test case -  List view of MCS alignment --- on multi-pages --"""
        try:
            pairList = [(tup[0].upper(), self.__oeMolD[tup[0].upper()], tup[1].upper(), self.__oeMolD[tup[1].upper()]) for tup in self.__pairIdList]
            oed = OeDepictMCSAlignMultiPage()
            oed.setDisplayOptions(
                labelAtomName=True,
                labelAtomCIPStereo=True,
                labelAtomIndex=False,
                labelBondIndex=False,
                highlightStyleFit="ballAndStickInverse",
                gridRows=3,
                gridCols=3,
                bondDisplayWidth=0.5,
            )
            oed.setPairMolList(pairList)
            imageFile = os.path.join(self.__workPath, "mcs-align-with-list-multi.pdf")
            aML = oed.alignOneWithListMulti(imagePath=imageFile)
            self.assertGreater(len(aML), 2)
            if aML:
                for (rCC, rAt, tCC, tAt) in aML:
                    logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testMCSAlignListDepict(self):
        """Test case -  List view of MCS alignment on single image -"""
        try:
            pairList = [(tup[0].upper(), self.__oeMolD[tup[0].upper()], tup[1].upper(), self.__oeMolD[tup[1].upper()]) for tup in self.__pairIdList]
            oed = OeDepictMCSAlignPage()
            oed.setDisplayOptions(
                imageSizeX=1500,
                imageSizeY=1500,
                labelAtomName=True,
                labelAtomCIPStereo=True,
                labelAtomIndex=False,
                labelBondIndex=False,
                highlightStyleFit="ballAndStickInverse",
                gridRows=3,
                gridCols=3,
                bondDisplayWidth=1.0,
            )
            oed.setPairMolList(pairList)
            imageFile = os.path.join(self.__workPath, "mcs-align-with-list-single.svg")
            aML = oed.alignOneWithList(imagePath=imageFile)
            self.assertGreater(len(aML), 2)
            if aML:
                for (rCC, rAt, tCC, tAt) in aML:
                    logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testMCSAlignListDepictSingle(self):
        """Test case -  View MCS alignment in multiple single image files."""
        try:
            pairList = [(tup[0].upper(), self.__oeMolD[tup[0].upper()], tup[1].upper(), self.__oeMolD[tup[1].upper()]) for tup in self.__pairIdList]
            oed = OeDepictMCSAlign()
            oed.setDisplayOptions(
                imageSizeX=500,
                imageSizeY=500,
                labelAtomName=True,
                labelAtomCIPStereo=True,
                labelAtomIndex=False,
                labelBondIndex=False,
                highlightStyleFit="ballAndStickInverse",
                bondDisplayWidth=1.0,
            )
            oed.setPairMolList(pairList, imageDirPath=self.__workPath, imageFilePrefix="single")
            aML = oed.alignOneWithList()
            self.assertGreater(len(aML), 2)
            if aML:
                for (rCC, rAt, tCC, tAt) in aML:
                    logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testMCSAlignPathListDepictSingle(self):
        """Test case -  View MCS alignments in single image files.

        Input details specified chemical component and image file paths.

        Image file paths must end with a recognized image format (e.g. svg, png, jpg)
        """
        try:
            # Use inverse  highlighting of matching/non-matching atoms/bonds -
            hOpt = "inverse"
            #
            oed = OeDepictMCSAlign()
            if hOpt == "inverse":
                oed.setDisplayOptions(
                    imageSizeX=500,
                    imageSizeY=500,
                    labelAtomName=True,
                    labelAtomCIPStereo=True,
                    labelAtomIndex=False,
                    labelBondIndex=False,
                    highlightStyleFit="ballAndStickInverse",
                    bondDisplayWidth=1.0,
                )
            elif hOpt == "match":
                oed.setDisplayOptions(
                    imageSizeX=500,
                    imageSizeY=500,
                    labelAtomName=True,
                    labelAtomCIPStereo=True,
                    labelAtomIndex=False,
                    labelBondIndex=False,
                    highlightStyleFit="ballAndStick",
                    bondDisplayWidth=1.0,
                )

            else:
                oed.setDisplayOptions(
                    imageSizeX=500,
                    imageSizeY=500,
                    labelAtomName=True,
                    labelAtomCIPStereo=True,
                    labelAtomIndex=False,
                    labelBondIndex=False,
                    highlightStyleFit="None",
                    bondDisplayWidth=1.0,
                )
            oed.setRefMol(self.__oeMolD[self.__refId], self.__refId)
            fitMolList = [self.__oeMolD[ccId.upper()] for ccId in self.__idList]
            oed.addFitMolList(fitMolList, suppressHydrogens=False, imageDirPath=self.__workPath, imageFilePrefix=hOpt)

            aML = oed.alignOneWithList()
            self.assertGreater(len(aML), 2)
            #
            # Write out atom correspondences --
            #
            if aML:
                for (rCC, rAt, tCC, tAt) in aML:
                    logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testSSAlignListDepictSingle(self):
        """Test case -  View substructure alignment in multiple single image files."""
        try:
            pairList = [(tup[0].upper(), self.__oeMolD[tup[0].upper()], tup[1].upper(), self.__oeMolD[tup[1].upper()]) for tup in self.__ssPairIdList]
            oed = OeDepictSubStructureAlign()
            oed.setDisplayOptions(
                imageSizeX=500,
                imageSizeY=500,
                labelAtomName=False,
                labelAtomCIPStereo=True,
                labelAtomIndex=False,
                labelBondIndex=False,
                highlightStyleFit="ballAndStick",
                highLightMatchColorFit="green",
                bondDisplayWidth=0.5,
            )
            oed.setPairMolList(pairList, imageDirPath=self.__workPath, imageFilePrefix="single")
            aML = oed.alignOneWithList()
            self.assertGreater(len(aML), 2)
            if aML:
                for (rCC, rAt, tCC, tAt) in aML:
                    logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def suiteAlignTests():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(OeDepictAlignTests("testMCSAlignAtomMap"))
    suiteSelect.addTest(OeDepictAlignTests("testMCSRelaxAlignPairDepict"))
    suiteSelect.addTest(OeDepictAlignTests("testMCSAlignPairDepict"))
    suiteSelect.addTest(OeDepictAlignTests("testMCSAlignFitListDepictMulti"))
    suiteSelect.addTest(OeDepictAlignTests("testMCSAlignPairListDepictPortrait"))
    suiteSelect.addTest(OeDepictAlignTests("testMCSAlignRnaPairListDepict"))
    suiteSelect.addTest(OeDepictAlignTests("testMCSAlignListMultiDepict"))
    suiteSelect.addTest(OeDepictAlignTests("testMCSAlignListDepict"))
    suiteSelect.addTest(OeDepictAlignTests("testMCSAlignPathListDepictSingle"))
    return suiteSelect


if __name__ == "__main__":
    #
    mySuite1 = suiteAlignTests()
    unittest.TextTestRunner(verbosity=2).run(mySuite1)
    #



##
# File:    ChemCompSearchWrapperTests.py
# Author:  J. Westbrook
# Date:    1-Oct-2020
# Version: 0.001
#
# Update:
#
#
##
"""
Tests for search modes using source molecular definitions coming from a search index.

"""
import logging
import os
import platform
import resource
import time
import unittest

from importlib.metadata import version as get_package_version
from rcsb.utils.chem.ChemCompSearchWrapper import ChemCompSearchWrapper
from rcsb.utils.io.FileUtil import FileUtil
from rcsb.utils.io.MarshalUtil import MarshalUtil

__version__ = get_package_version("rcsb.utils.chem")

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()
logger.setLevel(logging.INFO)


class ChemCompSearchWrapperTests(unittest.TestCase):
    def setUp(self):
        self.__startTime = time.time()
        self.__workPath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output", "CACHE")
        self.__buildTypeList = ["oe-iso-smiles", "oe-smiles", "cactvs-iso-smiles", "cactvs-smiles", "inchi"]
        #
        self.__mU = MarshalUtil(workPath=self.__cachePath)
        # Set the external environment for the wrapper class-
        self.__testFlagFull = False
        self.__testStash = False
        if self.__testFlagFull:
            os.environ["CHEM_SEARCH_CACHE_PATH"] = os.path.join(self.__cachePath)
            os.environ["CHEM_SEARCH_CC_PREFIX"] = "cc-full"
        else:
            os.environ["CHEM_SEARCH_CACHE_PATH"] = os.path.join(self.__cachePath)
            os.environ["CHEM_SEARCH_CC_PREFIX"] = "cc-abbrev"
        #
        self.__numMolsTest = 20
        logger.debug("Running tests on version %s", __version__)
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testAABuildConfiguration(self):
        """Test case - build configuration -"""
        try:
            ccsw = ChemCompSearchWrapper()
            ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif") if not self.__testFlagFull else None
            birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif") if not self.__testFlagFull else None
            ok = ccsw.setConfig(ccUrlTarget=ccUrlTarget, birdUrlTarget=birdUrlTarget)
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testAABuildDependencies(self):
        """Test case - build all dependencies (convenience method)"""
        try:
            ccsw = ChemCompSearchWrapper()
            ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif") if not self.__testFlagFull else None
            birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif") if not self.__testFlagFull else None
            ok = ccsw.buildDependenices(ccUrlTarget=ccUrlTarget, birdUrlTarget=birdUrlTarget)
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    @unittest.skip("Private test")
    def testAABuildDependenciesAndStash(self):
        """Test case - build, stash and restore dependencies -"""
        try:
            ccsw = ChemCompSearchWrapper()
            ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif") if not self.__testFlagFull else None
            birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif") if not self.__testFlagFull else None
            ok = ccsw.buildDependenices(ccUrlTarget=ccUrlTarget, birdUrlTarget=birdUrlTarget)
            self.assertTrue(ok)
            #
            if self.__testStash:
                url = "sftp://bl-east.rcsb.org"
                userName = ""
                pw = ""
                dirPath = "4-coastal"
                ok = ccsw.stashDependencies(url, dirPath, userName=userName, pw=pw)
                self.assertTrue(ok)
                #
                fileU = FileUtil()
                fileU.remove(self.__cachePath)
                #
                url = "http://bl-east.rcsb.org"
                ok = ccsw.restoreDependencies(url, dirPath)
                #
                fileU.remove(self.__cachePath)
                #
                url = "sftp://bl-east.rcsb.org"
                ok = ccsw.restoreDependencies(url, dirPath, userName=userName, pw=pw)
                self.assertTrue(ok)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testAReadConfig(self):
        """Test read/access configuration"""
        try:
            ccsw = ChemCompSearchWrapper()
            ok = ccsw.readConfig()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testBUpdateChemCompIndex(self):
        """Test update chemical component/Bird basic index."""
        try:
            ccsw = ChemCompSearchWrapper()
            ok = ccsw.readConfig()
            self.assertTrue(ok)
            ok = ccsw.updateChemCompIndex()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testCUpdateSearchIndex(self):
        """Test update search index."""
        try:
            ccsw = ChemCompSearchWrapper()
            ok = ccsw.readConfig()
            self.assertTrue(ok)
            ok = ccsw.updateSearchIndex()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testDUpdateSearchMoleculeProvider(self):
        """Test update of the search molecule provider."""
        try:
            ccsw = ChemCompSearchWrapper()
            ok = ccsw.readConfig()
            self.assertTrue(ok)
            ok = ccsw.updateSearchMoleculeProvider()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testEReloadSearchDatabase(self):
        """Test reload search databases."""
        try:
            ccsw = ChemCompSearchWrapper()
            ok = ccsw.readConfig()
            self.assertTrue(ok)
            ok = ccsw.reloadSearchDatabase()
            self.assertTrue(ok)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testZoomMatchDescriptor(self):
        """Test descriptor matching"""
        try:
            numMolsTest = self.__numMolsTest
            ccsw = ChemCompSearchWrapper()
            ok = ccsw.readConfig()
            self.assertTrue(ok)
            ok = ccsw.updateChemCompIndex(useCache=True)
            self.assertTrue(ok)
            ccIdx = ccsw.getChemCompIndex()
            ok = ccsw.reloadSearchDatabase()
            self.assertTrue(ok)
            #
            logger.debug("ccIdx (%d) keys %r entry %r", len(ccIdx), list(ccIdx.keys())[:10], ccIdx["000"])
            #
            logger.info("Dependencies loaded - Starting search test scan of (limit=%r)", numMolsTest)
            for ii, (ccId, ccD) in enumerate(ccIdx.items(), 1):
                if numMolsTest and ii > numMolsTest:
                    break
                for buildType in self.__buildTypeList:
                    if buildType in ccD:
                        startTime = time.time()
                        retStatus, ssL, fpL = ccsw.searchByDescriptor(ccD[buildType], buildType, matchOpts="graph-relaxed")
                        mOk = self.__resultContains(ccId, ssL)
                        self.assertTrue(mOk)
                        #
                        ssCcIdList = list(set([t.ccId.split("|")[0] for t in ssL]))
                        fpCcIdList = list(set([t.ccId.split("|")[0] for t in fpL]))
                        logger.info(
                            "%s (%d) for buildType %s (%d/%d) (%r) %r (%.4f secs)",
                            ccId,
                            ii,
                            buildType,
                            len(ssCcIdList),
                            len(fpCcIdList),
                            mOk and retStatus == 0,
                            ssCcIdList,
                            time.time() - startTime,
                        )
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testZoomSubStructSearch(self):
        """Test substructure search"""
        try:
            numMolsTest = self.__numMolsTest
            ccsw = ChemCompSearchWrapper()
            ok = ccsw.readConfig()
            self.assertTrue(ok)
            ok = ccsw.updateChemCompIndex(useCache=True)
            self.assertTrue(ok)
            ccIdx = ccsw.getChemCompIndex()
            ok = ccsw.updateSearchIndex(useCache=True)
            self.assertTrue(ok)
            ok = ccsw.reloadSearchDatabase()
            self.assertTrue(ok)
            #
            logger.debug("ccIdx (%d) keys %r entry %r", len(ccIdx), list(ccIdx.keys())[:10], ccIdx["000"])
            #
            logger.info("Dependencies loaded - Starting search test scan of (limit=%r)", numMolsTest)
            for ii, (ccId, ccD) in enumerate(ccIdx.items(), 1):
                if numMolsTest and ii > numMolsTest:
                    break
                for buildType in self.__buildTypeList:
                    if buildType in ccD:
                        startTime = time.time()
                        retStatus, ssL, _ = ccsw.searchByDescriptor(ccD[buildType], buildType, matchOpts="sub-struct-graph-relaxed")
                        if retStatus == -100:
                            logger.warning("Descriptor error continuing...")
                            continue
                        mOk = self.__resultContains(ccId, ssL)
                        self.assertTrue(mOk)
                        #
                        ssCcIdList = list(set([t.ccId.split("|")[0] for t in ssL]))
                        logger.info(
                            "%s (%d) for buildType %s (%d) (%r) %r (%.4f secs)",
                            ccId,
                            ii,
                            buildType,
                            len(ssCcIdList),
                            mOk and retStatus == 0,
                            ssCcIdList,
                            time.time() - startTime,
                        )
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testZoomMatchFormula(self):
        """Test formula matching"""
        try:
            numMolsTest = self.__numMolsTest
            ccsw = ChemCompSearchWrapper()
            ok = ccsw.readConfig()
            self.assertTrue(ok)
            ok = ccsw.updateChemCompIndex(useCache=True)
            self.assertTrue(ok)
            ccIdx = ccsw.getChemCompIndex()
            #
            logger.debug("ccIdx (%d) keys %r entry %r", len(ccIdx), list(ccIdx.keys())[:10], ccIdx["000"])
            #
            logger.info("Dependencies loaded - Starting formula test scan of (limit=%r)", numMolsTest)
            for ii, (ccId, idxD) in enumerate(ccIdx.items(), 1):
                if numMolsTest and ii > numMolsTest:
                    break
                #
                startTime = time.time()
                elementRangeD = {el: {"min": eCount, "max": eCount} for el, eCount in idxD["type-counts"].items()}
                retStatus, rL = ccsw.matchByFormulaRange(elementRangeD, ccId)
                mOk = self.__resultContains(ccId, rL)
                self.assertTrue(mOk)
                logger.info(
                    "%s (%d) (%d) (%r) (%.4f secs)",
                    ccId,
                    ii,
                    len(rL),
                    mOk and retStatus == 0,
                    time.time() - startTime,
                )
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    @unittest.skip("Private test")
    def testZoomFingerprintRepeat(self):
        """Test substructure search"""
        try:
            ccsw = ChemCompSearchWrapper()
            ok = ccsw.readConfig()
            self.assertTrue(ok)
            ok = ccsw.updateChemCompIndex(useCache=True)
            self.assertTrue(ok)
            ccIdx = ccsw.getChemCompIndex()
            ok = ccsw.updateSearchIndex(useCache=True)
            self.assertTrue(ok)
            ok = ccsw.reloadSearchDatabase()
            self.assertTrue(ok)
            #
            logger.debug("ccIdx (%d)", len(ccIdx))
            #
            fpL = []
            descr = "InChI=1S/C9H15N5O3/c1-3(15)6(16)4-2-11-7-5(12-4)8(17)14-9(10)13-7/h3-4,6,12,15-16H,2H2,1H3,(H4,10,11,13,14,17)/t3-,4-,6-/m1/s1"
            for ii in range(100):
                startTime = time.time()
                retStatus, _, fpL = ccsw.searchByDescriptor(descr, "InChI", matchOpts="fingerprint-similarity")
                if retStatus == -100:
                    logger.warning("Descriptor error continuing...")
                    continue
                rD = {}
                for mr in fpL:
                    ccId = mr.ccId.split("|")[0]
                    rD[ccId] = max(rD[ccId], mr.fpScore) if ccId in rD else mr.fpScore
                rTupL = sorted(rD.items(), key=lambda kv: kv[1], reverse=True)
                rL = [rTup[0] for rTup in rTupL]
                scoreL = [rTup[1] for rTup in rTupL]
                logger.info("%4d (%3d) %r (%.4f secs)", ii, len(rL), retStatus == 0, time.time() - startTime)
            logger.info("rL %r", rL)
            logger.info("scoreL %r", scoreL)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def __resultContains(self, ccId, matchResultList):
        for matchResult in matchResultList:
            if ccId in matchResult.ccId:
                return True
        return False


def fullSearchTests():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ChemCompSearchWrapperTests("testMatchSmiles"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = fullSearchTests()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    BatchChemSearchWrapperTests.py
# Author:  jdw
# Date:    1-Mar-2021
# Version: 0.001
#
# Updates:
#
##
"""
Wrapper for batch chemical search operations.
"""

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"

import logging
import os
import time

from collections import namedtuple

from rcsb.utils.chem.ChemCompSearchWrapper import ChemCompSearchWrapper
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil

logger = logging.getLogger(__name__)

BatchResults = namedtuple("BatchResults", "queryId query queryType matchOpts ccId fpScore", defaults=(None,) * 6)


class BatchChemSearchWorker(object):
    def __init__(self, ccsW, **kwargs):
        self.__ccsW = ccsW
        _ = kwargs

    def searchDescriptorList(self, dataList, procName, optionsD, workingDir):
        """Multiprocessing method template to execute batch descriptor query"""
        _ = optionsD
        _ = workingDir
        queryType = optionsD.get("queryType", "SMILES")
        matchOpts = optionsD.get("matchOpts", "graph-relaxed")
        successList = []
        failList = []
        retList = []
        diagList = []
        #
        try:
            retList, failList = self.__queryDescriptorList(procName, dataList, queryType, matchOpts=matchOpts)
            successList = sorted(set(dataList) - set(failList))
            if failList:
                logger.info("%s returns %d definitions with failures", procName, len(failList))

            logger.debug("%s built %d search candidates from %d definitions with failures %d", procName, len(retList), len(dataList), len(failList))
        except Exception as e:
            logger.exception("Failing %s for %d data items %s", procName, len(dataList), str(e))
        #
        return successList, retList, diagList

    def __queryDescriptorList(self, procName, queryPairList, queryType, matchOpts="graph-strict"):
        """Internal method to perform description search for the input descriptor/identifier list"""
        rL = []
        fL = []
        queryId = None
        try:
            for queryPair in queryPairList:
                queryId, descr = queryPair
                #
                retStatus, mL, _ = self.__ccsW.searchByDescriptor(descr, queryType, matchOpts=matchOpts)
                #
                if retStatus == 0 and len(mL) > 0:
                    rL.append((queryId, descr, queryType, mL))
                else:
                    fL.append(queryPair)
            #
        except Exception as e:
            logger.exception("%s %r failing with %s", procName, queryId, str(e))
        #
        return rL, fL


class BatchChemSearch(object):
    """Wrapper for batch chemical search operations."""

    def __init__(self, **kwargs):
        """Wrapper class for batch chemical search/depiction operations.

        Path and prefix data for wrapper class may be set as keyword arguments
        as environmental variables.

        Args:
            ccUrlTarget (str, optional): path to concatenated chemical component definition file. Defaults to public data file.
            birdUrlTarget (str, optional): path to the concatenated BIRD definition file.  Defaults to public data file.
            cachePath (str): path to top-level cache directory used to store search index file dependencies
                             (default environment variable CHEM_SEARCH_CACHE_PATH or ".")
            numProc (int): multi-process cores to reserve. Default to 6.
            chunkSize (int): multi-process batch size.  Defaults to 50.
        """
        self.__startTime = time.time()
        #
        self.__useCache = kwargs.get("useCache", True)
        self.__numProc = kwargs.get("numProc", 6)
        self.__chunkSize = kwargs.get("chunkSize", 50)
        #
        self.__ccUrlTarget = kwargs.get("ccUrlTarget", None)
        self.__birdUrlTarget = kwargs.get("birdUrlTarget", None)
        self.__ccFileNamePrefix = kwargs.get("ccFileNamePrefix", os.environ.get("CHEM_SEARCH_CC_PREFIX", "cc-full"))
        #
        self.__cachePath = kwargs.get("cachePath", os.environ.get("CHEM_SEARCH_CACHE_PATH", "."))
        # ---
        self.__mU = MarshalUtil(workPath=self.__cachePath)
        # ---
        self.__ccsw = self.__reload()
        #

    def __reload(self):
        ccsw = ChemCompSearchWrapper(cachePath=self.__cachePath)
        ok1 = ccsw.setConfig(
            self.__ccUrlTarget, self.__birdUrlTarget, ccFileNamePrefix=self.__ccFileNamePrefix, useCache=self.__useCache, numProc=self.__numProc, maxChunkSize=self.__chunkSize
        )
        ok2 = ccsw.readConfig()
        ok3 = ccsw.reloadSearchDatabase()
        return ccsw if ok1 and ok2 and ok3 else None

    def testCache(self):
        return self.__ccsw is not None

    def fetchDescriptorList(self, filePath, fmt="tdd", swap=False):
        """Fetch list of descriptor and identifier code pairs.

        Args:
            filePath (str): input descriptor/identifier file path
            fmt (str, optional):  tab delimited 'tdd' or comma-separated values 'csv'. Defaults to 'tdd'.
            swap (bool, optional): swap order of input fields. Defaults to False.

        Returns:
            (list): [description]
        """
        rL = []
        pairL = self.__mU.doImport(filePath, fmt=fmt, rowFormat="list")
        if swap:
            for pair in pairL:
                if len(pair) != 2:
                    continue
                rL.append((pair[1], pair[0]))
        return rL

    def splitSmiles(self, pairList):
        rL = []
        for pair in pairList:
            queryId, descr = pair
            if "." in descr:
                ff = descr.split(".")
                descr = max(ff, key=len)
                rL.append((queryId, descr))
            else:
                rL.append(pair)
        return rL

    def storeMatchList(self, filePath, matchList):
        mL = []
        # namedtuple("BatchResults", "queryId query queryType matchOpts ccId fpScore", defaults=(None,) * 6)
        for match in matchList:
            mL.append({"queryId": match.queryId, "query": match.query, "queryType": match.queryType, "matchOpts": match.matchOpts, "ccId": match.ccId, "fpScore": match.fpScore})
        return self.__mU.doExport(filePath, mL, fmt="json")

    def fetchMatchList(self, filePath):
        mDL = self.__mU.doImport(filePath, fmt="json")
        mL = []
        for mD in mDL:
            mL.append(BatchResults(queryId=mD["queryId"], query=mD["query"], queryType=mD["queryType"], matchOpts=mD["matchOpts"], ccId=mD["ccId"], fpScore=mD["fpScore"]))
        return mL

    def doQuery(self, queryPairList, queryType, matchOpts="graph-strict"):
        """Batch descriptor query.

        Args:
            queryPairList (list): [description]
            queryType ([type]): [description]
            matchOpts (str, optional): [description]. Defaults to "graph-strict".

        Returns:
            (list): [BatchResults(), BatchResults(), ...]
        """

        rL = []
        try:
            if queryType.upper() in ["SMILES", "INCHI"]:
                rL = self.__descriptorQueryMulti(queryPairList, queryType, matchOpts, numProc=self.__numProc, chunkSize=self.__chunkSize)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        #
        return rL

    def __descriptorQueryMulti(self, queryPairList, queryType, matchOpts, numProc=2, chunkSize=50):
        """Internal method to invoke descriptor query in multiprocess mode

        Args:
            queryPairList (list): [(identifer, descriptor), ... ]
            queryType (str): SMILES|InChI
            matchOpts (str): match criteria (e.g., graph-strict)
            numProc (int, optional): number of multiprocess cores. Defaults to 2.
            chunkSize (int, optional): multiprocess batch size. Defaults to 50.

        Returns:
            (list): [BatchResults(), ...]
        """
        logger.info("Input %r query length %d using %s numProc %d", queryType, len(queryPairList), matchOpts, numProc)
        rWorker = BatchChemSearchWorker(self.__ccsw)
        mpu = MultiProcUtil(verbose=True)
        optD = {"matchOpts": matchOpts, "queryType": queryType}
        mpu.setOptions(optD)
        mpu.set(workerObj=rWorker, workerMethod="searchDescriptorList")
        ok, failList, resultList, _ = mpu.runMulti(dataList=queryPairList, numProc=numProc, numResults=1, chunkSize=chunkSize)
        if failList:
            logger.debug("Search completed with failures (%d)", len(failList))
        logger.info("Multi-proc status %r failures %r result length %r", ok, len(failList), len(resultList[0]))
        #
        rL = []
        for tup in resultList[0]:
            for mr in tup[3]:
                rL.append(BatchResults(queryId=tup[0], query=tup[1], queryType=tup[2], matchOpts=matchOpts, ccId=mr.ccId, fpScore=mr.fpScore))
        #
        logger.info("Multi-proc status %r failures %r result length %r", ok, len(failList), len(rL))
        #
        return rL



##
# File:    OeMolecularFactoryTests.py
# Author:  jdw
# Date:    28-Oct-2019
# Version: 0.001
#
# Updates:
#
##
"""
A collection of tests of OeMolecularFactory to compare assigned and computed features.

"""
import logging
import os
import platform
import resource
import time
import unittest

from rcsb.utils.chem.ChemAxonDescriptorProvider import ChemAxonDescriptorProvider
from rcsb.utils.chem.ChemCompMoleculeProvider import ChemCompMoleculeProvider
from rcsb.utils.chem.MoleculeAnnotationsCompare import MoleculeAnnotationsCompare
from rcsb.utils.chem.OeDepictAlign import OeDepictMCSAlignPage
from rcsb.utils.chem.OeMoleculeFactory import OeMoleculeFactory
from rcsb.utils.chem.PdbxChemComp import PdbxChemCompIt

logging_level = logging.INFO
logging_fmt = "%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s"
try:
    root_logger = logging.getLogger()
    root_logger.setLevel(logging_level)
    root_handler = root_logger.handlers[0]
    root_handler.setFormatter(logging.Formatter(logging_fmt))
except IndexError:
    logging.basicConfig(level=logging_level, format=logging_fmt)

logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class OeMolecularFactoryTests(unittest.TestCase):
    skipFlag = True

    def setUp(self):
        self.__startTime = time.time()
        #
        self.__workPath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__cachePath = os.path.join(HERE, "test-output")
        self.__ccUrlTarget = os.path.join(self.__dataPath, "components-abbrev.cif")
        self.__birdUrlTarget = os.path.join(self.__dataPath, "prdcc-abbrev.cif")
        self.__molLimit = None
        logger.info("Setup completed")
        print("log level %r (logging.INFO %r)" % (logger.level, logging.INFO))
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def __getChemCompDefs(self, molLimit=None):
        ccMolD = {}
        try:
            logger.info("Building molecule cache")
            useCache = True
            ccFileNamePrefix = "cc-abbrev"
            ccmP = ChemCompMoleculeProvider(
                ccUrlTarget=self.__ccUrlTarget,
                birdUrlTarget=self.__birdUrlTarget,
                cachePath=self.__cachePath,
                useCache=useCache,
                ccFileNamePrefix=ccFileNamePrefix,
                molLimit=molLimit,
            )
            ok = ccmP.testCache(minCount=molLimit)
            self.assertTrue(ok)
            logger.info("Build molecule cache (%r)", ok)
            ccMolD = ccmP.getMolD()
            logger.info("ccMolD (%d)", len(ccMolD))
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ccMolD

    def testBuildRelated(self):
        """Test build molecules chemical definitions -"""
        try:
            logger.info("Starting")
            ccMolD = self.__getChemCompDefs()
            oemf = OeMoleculeFactory(quietMode=True)
            relD = {}
            for ccId, ccObj in list(ccMolD.items())[:100]:
                # ----
                tId = oemf.setChemCompDef(ccObj)
                self.assertEqual(tId, ccId)
                tD = oemf.buildRelated(limitPerceptions=False)
                logger.info("%s related molecular forms %d", ccId, len(tD))
                relD.update(tD)
            logger.info("Total molecular forms (%d)", len(relD))
            # ----
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testBuildRelatedExtra(self):
        """Test build molecules chemical definitions including extra chemaxon descriptors"""

        try:
            caxP = ChemAxonDescriptorProvider(
                ccUrlTarget=self.__ccUrlTarget, birdUrlTarget=self.__birdUrlTarget, cachePath=self.__cachePath, useCache=True, ccFileNamePrefix="cc-abbrev"
            )
            ok = caxP.testCache()
            self.assertTrue(ok)
            descrD = caxP.getDescriptorIndex()
            #
            ccMolD = self.__getChemCompDefs()
            oemf = OeMoleculeFactory(quietMode=True)
            relD = {}
            for ccId, ccObj in list(ccMolD.items())[:100]:
                # ----
                tId = oemf.setChemCompDef(ccObj)
                self.assertEqual(tId, ccId)
                #
                oemf.clearExternalDescriptors()
                for smi in descrD[ccId] if ccId in descrD else []:
                    oemf.addExternalDescriptor("smiles", smi, "chemaxon-smiles")
                #
                tD = oemf.buildRelated(limitPerceptions=False)
                logger.info("%s related molecular forms %d", ccId, len(tD))
                relD.update(tD)
            logger.info("Total molecular forms (%d)", len(relD))

            # ----
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    @unittest.skipIf(skipFlag, "Troubleshooting test")
    def testSelfConsistency(self):
        """Compare constructed molecule with underlying chemical definitions -"""
        try:
            failL = []
            ccMolD = self.__getChemCompDefs()
            #
            # molBuildTypeL = ["model-xyz", "ideal-xyz", None]
            # molBuildTypeL = [None]
            #
            oemf = OeMoleculeFactory()
            macmp = MoleculeAnnotationsCompare()

            limitPerceptions = False
            # buildTypeRef = "oe-iso-smiles"
            buildTypeRef = "model-xyz"
            filterHydrogens = False
            if buildTypeRef in ["oe-iso-smiles", "oe-smiles", "cactvs-smiles", "cactvs-iso-smiles", "acdlabs-smiles", "inchi"]:
                filterHydrogens = True
            #
            for ccId, ccObj in ccMolD.items():
                # ----
                tId = oemf.setChemCompDef(ccObj)
                self.assertEqual(tId, ccId)
                ok = oemf.build(molBuildType=buildTypeRef, limitPerceptions=limitPerceptions, normalize=False)
                if not ok:
                    logger.info("Build using %r failed for %s", buildTypeRef, ccId)
                    continue
                #
                doTautomers = False
                if doTautomers:
                    tautomerMolL = oemf.getTautomerMolList()
                    logger.info("%s number reasonable tautomers %d", ccId, len(tautomerMolL))
                #
                refFD = macmp.getChemCompFeatures(ccObj, descriptorProgram="OPENEYE", filterHydrogens=filterHydrogens)
                tstFD = oemf.getOeMoleculeFeatures(filterHydrogens=filterHydrogens)
                # logger.info("tstFD %r", tstFD)
                ok, retCmp = macmp.compare(refFD, tstFD, tstInfo="Openeye ISO SMILES")
                if not ok:
                    logger.info("Comparison failed build type %r and %r", buildTypeRef, ccId)
                    logger.debug(
                        "diff -> atomatic atoms %r stereo atoms %r bond types %r aromatic bonds %r",
                        retCmp.difAromaticAtoms,
                        retCmp.difStereoAtoms,
                        retCmp.difTypeBonds,
                        retCmp.difAromaticBonds,
                    )
                    failL.append(ccId)
                #
            logger.info("Failures (%d) %r: ", len(failL), failL)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testRoundTrip(self):
        """Round trip smiles comparisons -"""
        try:
            ccMolD = self.__getChemCompDefs()
            # useCache = True
            # quietFlag = False
            # molBuildTypeL = ["model-xyz", "ideal-xyz", None]
            # molBuildTypeL = [None]
            buildTypeRef = "oe-iso-smiles"
            oemf1 = OeMoleculeFactory()
            oemf2 = OeMoleculeFactory()
            #
            for ccId, ccObj in ccMolD.items():
                # ----
                ccIt = iter(PdbxChemCompIt(ccObj))
                cc = next(ccIt)
                formula = cc.getFormulaWithCharge()
                # ccId = cc.getId()
                ccName = cc.getName()
                ifCharge = cc.getFormalChargeAsInt()
                isAmbiguous = cc.getAmbiguousFlag() in ["Y", "y"]
                isCurrent = cc.getReleaseStatus() in ["REL"]
                logger.debug("%s name %r formula %r charge %d", ccId, ccName, formula, ifCharge)
                # ----
                ccId = oemf1.setChemCompDef(ccObj)
                ok = oemf1.build(molBuildType=buildTypeRef, limitPerceptions=False)
                if not ok:
                    logger.info("Build using %r failed for %s (ambiguous flag %r current %r)", buildTypeRef, ccId, isAmbiguous, isCurrent)
                #
                isDiff = False
                #
                if isDiff:
                    genIsoSmi = oemf1.getCanSMILES()
                    oemf2 = OeMoleculeFactory()
                    oemf2.setDescriptor(genIsoSmi, "oe-iso-smiles", ccId)
                    oemf2.build(molBuildType="oe-iso-smiles", limitPerceptions=False)
                    regenIsoSmi = oemf2.getIsoSMILES()
                    if genIsoSmi != regenIsoSmi:
                        logger.info("%s  regenerated ISOSMILES differ \n -- INP: %s\n -- OUT: %s", ccId, genIsoSmi, regenIsoSmi)

                    oed = OeDepictMCSAlignPage()
                    oed.setDisplayOptions(
                        labelAtomName=True,
                        labelAtomCIPStereo=True,
                        labelAtomIndex=False,
                        labelBondIndex=False,
                        labelBondCIPStereo=True,
                        highlightStyleFit="ballAndStickInverse",
                        highLightNotMatchColorRef="pink",
                        bondDisplayWidth=0.5,
                    )
                    oed.setRefMol(oemf1.getGraphMol(), ccId)
                    oed.setFitMol(oemf2.getGraphMol(), ccId)
                    imgPath = os.path.join(self.__workPath, "compare-assigned-" + ccId + "-calc-" + ccId + ".svg")
                    logger.info("Using image path %r", imgPath)
                    aML = oed.alignPair(imagePath=imgPath)
                    if aML:
                        for (rCC, rAt, tCC, tAt) in aML:
                            logger.info("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)
                else:
                    logger.debug("%s matched all cases", ccId)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testBuilders(self):
        try:
            ccMolD = self.__getChemCompDefs()
            quietFlag = False
            molBuildTypeL = ["model-xyz", "ideal-xyz"]
            for molBuildType in molBuildTypeL:
                oemf = OeMoleculeFactory()
                if quietFlag:
                    oemf.setQuiet()
                #
                eCount = 0
                for tId, ccObj in ccMolD.items():
                    ccId = oemf.setChemCompDef(ccObj)
                    self.assertEqual(tId, ccId)
                    logger.debug("Building %s using molBuildType %r", ccId, molBuildType)
                    if ccId:
                        ok = oemf.build(molBuildType=molBuildType)
                        logger.debug("Comparing built component %s using molBuildType %r", ccId, molBuildType)
                        # ok = oemf.compare()
                        ok = True
                        if not ok:
                            logger.info("Failing on %s molBuildType %r", ccId, molBuildType)
                            eCount += 1
                        # self.assertTrue(ok)
                    else:
                        logger.error("Cannot process %r", ccObj.getName())
                logger.info("Processed %d components molBuildType %r errors %d", len(ccMolD), molBuildType, eCount)
                #
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def suiteOeMoleculeFactoryTests():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(OeMolecularFactoryTests("testBuildRelated"))
    return suiteSelect


if __name__ == "__main__":
    #
    mySuite1 = suiteOeMoleculeFactoryTests()
    unittest.TextTestRunner(verbosity=2).run(mySuite1)
    #



##
# File:    IoUtilsTests.py
# Author:  J. Westbrook
# Date:    1-Oct-2019
# Version: 0.001
#
# Update:
#
#
##
"""
Tests for utilities to read and write chemical component definitions.

"""
import logging
import os
import time
import unittest

from importlib.metadata import version as get_package_version
from rcsb.utils.chem.IoUtils import IoUtils

__version__ = get_package_version("rcsb.utils.chem")

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()
logger.setLevel(logging.INFO)


class IoUtilsTests(unittest.TestCase):
    skipFlag = True

    def setUp(self):
        self.__workPath = os.path.join(HERE, "test-output")
        self.__dataPath = os.path.join(HERE, "test-data")
        self.__startTime = time.time()
        self.__exportFlag = True
        logger.debug("Running tests on version %s", __version__)
        logger.info("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testSdfGenerator(self):
        ioU = IoUtils()
        sdfL = []
        rdCcObjL = ioU.getComponentDefinitions(os.path.join(self.__dataPath, "components-abbrev.cif"))
        self.assertGreaterEqual(len(rdCcObjL), 4)
        for rdCcObj in rdCcObjL:
            ok, sdfS, _ = ioU.makeSdf(rdCcObj, molBuildType="ideal-xyz")
            if not ok:
                continue
            if self.__exportFlag:
                fp = os.path.join(self.__workPath, rdCcObj.getName() + ".sdf")
                with open(fp, "w", encoding="utf-8") as ofh:
                    ofh.write("%s" % sdfS)
            sdfL.append(sdfS)
        #

    @unittest.skipIf(skipFlag, "Long test")
    def testSdfBulkGenerator(self):
        ioU = IoUtils()
        sdfL = []
        rdCcObjL = ioU.getComponentDefinitions(os.path.join(self.__dataPath, "components.cif.gz"))
        self.assertGreaterEqual(len(rdCcObjL), 4)
        for rdCcObj in rdCcObjL[:500]:
            ok, sdfS, _ = ioU.makeSdf(rdCcObj, molBuildType="ideal-xyz")
            if not ok:
                continue
            sdfL.append(sdfS)
        #
        fp = os.path.join(self.__workPath, "components.sdf")
        with open(fp, "w", encoding="utf-8") as ofh:
            ofh.write("%s" % "\n".join(sdfL))


def sdfWriterSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(IoUtilsTests("testSdfGenerator"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = sdfWriterSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    ChemCompDepictWrapper.py
# Author:  jdw
# Date:    9-Mar-2020
# Version: 0.001
#
# Updates:
#  24-Jun-2020 jdw make api follow the style of ChemCompSearchWrapper()
##
"""
Wrapper for chemical component depiction operations.
"""

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"

import logging
import os
import platform
import resource
import time

from rcsb.utils.chem.ChemCompSearchWrapper import ChemCompSearchWrapper
from rcsb.utils.chem.OeDepict import OeDepict
from rcsb.utils.chem.OeDepictAlign import OeDepictMCSAlignPage
from rcsb.utils.chem.OeIoUtils import OeIoUtils
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.io.SingletonClass import SingletonClass

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logger = logging.getLogger(__name__)


class ChemCompDepictWrapper(SingletonClass):
    """Wrapper for chemical component depiction operations."""

    def __init__(self):
        self.__startTime = time.time()
        # ---
        self.__workPath = "."
        self.__mU = MarshalUtil(workPath=self.__workPath)
        self.__configD = None
        self.__cachePath = None
        # ---
        self.__statusDescriptorError = -100
        self.__searchError = -200
        self.__searchSuccess = 0
        self.__imageCount = 0

    def readConfig(self, resetImagePath=True):
        #
        ok = False
        try:
            self.__cachePath = os.environ.get("CHEM_DEPICT_CACHE_PATH", ".")
            configFileName = os.environ.get("CHEM_DEPICT_CONFIG_FILE_NAME", "depict-config.json")
            #
            configFilePath = os.path.join(self.__cachePath, "config", configFileName)
            configD = {}
            if self.__mU.exists(configFilePath):
                configD = self.__mU.doImport(configFilePath, fmt="json")
            logger.debug("configD: %r", configD)
            if configD and (len(configD) >= 2) and float(configD["versionNumber"]) > 0.1:
                logger.info("Read version %r sections %r from %s", configD["versionNumber"], list(configD.keys()), configFilePath)
                ok = True
                #
                if resetImagePath:
                    # Allow the configuration to be relocatable.
                    tS = configD["imageDir"] if "imageDir" in configD else "images"
                    configD["imageDirPath"] = os.path.join(self.__cachePath, tS)
                    configD["versionNumber"] = "0.2"
            else:
                # Handle missing config for now
                configD["imageDir"] = "images"
                configD["imageDirPath"] = os.path.join(self.__cachePath, configD["imageDir"])
                logger.warning("Reading config file fails from path %r", configFilePath)
                logger.warning("Using config %r", configD)
                ok = True
            #
            self.__configD = configD
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            ok = False
        return ok

    def setConfig(self, cachePath, **kwargs):
        """Provide dependencies for rebuilding depict file dependencies.

        Args:
            cachePath (str): path to cache data files.

            Other options are propagated to configurations of the wrapped classes in __bootstrapDepictConfig()

        """
        self.__configD = self.__makeBootstrapDepictConfig(cachePath, **kwargs)
        return len(self.__configD) >= 2

    def __makeBootstrapDepictConfig(self, cachePath, **kwargs):
        """Create depict configuration bootstrap file"""
        configD = {}
        try:
            storeConfig = kwargs.get("storeConfig", True)
            os.environ["CHEM_DEPICT_CACHE_PATH"] = os.path.join(cachePath)
            configDirPath = os.path.join(cachePath, "config")
            configFilePath = os.path.join(configDirPath, "depict-config.json")
            #
            logger.info("Updating depict configuration using %s", configFilePath)
            #
            imageDirPath = os.path.join(cachePath, "images")
            self.__mU.mkdir(imageDirPath)
            configD = {"versionNumber": 0.20, "imageDir": "images"}
            if storeConfig:
                self.__mU.mkdir(configDirPath)
                self.__mU.doExport(configFilePath, configD, fmt="json", indent=3)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return configD
        #

    def setImageCount(self, imageCount):
        self.__imageCount = imageCount

    def getImageCount(self):
        return self.__imageCount

    def __makeImagePath(self):
        imageDirPath = self.__configD["imageDirPath"] if self.__configD and "imageDirPath" in self.__configD else "."
        fileRotateIncrement = self.__configD["fileRotateIncrement"] if self.__configD and "fileRotateIncrement" in self.__configD else 50
        ic = self.__imageCount % fileRotateIncrement
        imagePath = os.path.join(imageDirPath, "image-%s.svg" % ic)
        return imagePath

    def depictMolecule(self, identifier, identifierType, imagePath=None, **kwargs):
        """Create depiction from InChI, SMILES descriptors or PDB identifier."""
        try:
            imagePath = imagePath if imagePath else self.__makeImagePath()
            oeMol = None
            oeio = OeIoUtils()
            if identifierType.lower() in ["smiles"]:
                oeMol = oeio.smilesToMol(identifier)
            elif identifierType.lower() in ["inchi"]:
                oeMol = oeio.inchiToMol(identifier)
            elif identifierType.lower() in ["identifierpdb"]:
                ccsw = ChemCompSearchWrapper()
                oesmP = ccsw.getSearchMoleculeProvider()
                oeMol = oesmP.getMol(identifier)
            #
            ok = self.__depictOne(oeMol, imagePath, **kwargs)
            return imagePath if ok else None
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return None

    def __depictOne(self, oeMol, imagePath, **kwargs):
        """Single

        Args:
            oeMol (object): instance of an OE graph molecule
            imagePath (string): file path for image

        Returns:
            bool: True for success or False otherwise
        """
        try:
            title = kwargs.get("title", None)
            oed = OeDepict()
            oed.setMolTitleList([("Target", oeMol, title)])

            # ---
            bondDisplayWidth = 10.0
            numAtoms = oeMol.NumAtoms()
            if numAtoms > 100 and numAtoms <= 200:
                bondDisplayWidth = 6.0
            elif numAtoms > 200:
                bondDisplayWidth = 4.0
            # ---
            oed.setDisplayOptions(
                imageSizeX=kwargs.get("imageSizeX", 2500),
                imageSizeY=kwargs.get("imageSizeX", 2500),
                labelAtomName=kwargs.get("labelAtomName", False),
                labelAtomCIPStereo=kwargs.get("labelAtomCIPStereo", True),
                labelAtomIndex=kwargs.get("labelAtomIndex", False),
                labelBondIndex=kwargs.get("labelBondIndex", False),
                labelBondCIPStereo=kwargs.get("labelBondCIPStereo", True),
                cellBorders=kwargs.get("cellBorders", True),
                bondDisplayWidth=bondDisplayWidth,
            )
            oed.setGridOptions(rows=1, cols=1, cellBorders=False)
            oed.prepare()
            oed.write(imagePath)
            self.__imageCount += 1
            return True
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return False

    def status(self):
        unitS = "MB" if platform.system() == "Darwin" else "GB"
        rusageMax = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        logger.info("Maximum resident memory size %.4f %s", rusageMax / 10 ** 6, unitS)
        endTime = time.time()
        logger.info("Status at %s (up %.4f seconds)", time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def alignMoleculePair(self, refIdentifier, refIdentifierType, fitIdentifier, fitIdentifierType, imagePath=None, **kwargs):
        """Create aligned depiction for a target molecule InChI, SMILES descriptors or PDB identifier."""
        try:
            imagePath = imagePath if imagePath else self.__makeImagePath()
            oeio = OeIoUtils()
            ccsw = ChemCompSearchWrapper()
            oesmP = ccsw.getSearchMoleculeProvider()
            oeMolRef = oeMolFit = None
            # ---
            if refIdentifierType.lower() in ["smiles"]:
                oeMolRef = oeio.smilesToMol(refIdentifier)
            elif refIdentifierType.lower() in ["inchi"]:
                oeMolRef = oeio.inchiToMol(refIdentifier)
            elif refIdentifierType.lower() in ["identifierpdb"]:
                oeMolRef = oesmP.getMol(refIdentifier)
            #
            if fitIdentifierType.lower() in ["smiles"]:
                oeMolFit = oeio.smilesToMol(fitIdentifier)
            elif fitIdentifierType.lower() in ["inchi"]:
                oeMolFit = oeio.inchiToMol(fitIdentifier)
            elif fitIdentifierType.lower() in ["identifierpdb"]:
                oeMolFit = oesmP.getMol(fitIdentifier)
            # ---
            logger.info("oeMolRef atoms %r", oeMolRef.NumAtoms())
            logger.info("oeMolFit atoms %r", oeMolFit.NumAtoms())

            displayIdRef = "Ref"
            displayIdFit = "Fit"
            ok = self.__depictAlignedPair(oeMolRef, displayIdRef, oeMolFit, displayIdFit, imagePath, **kwargs)
            return imagePath if ok else None
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return None

    def __depictAlignedPair(self, oeMolRef, displayIdRef, oeMolFit, displayIdFit, imagePath, **kwargs):
        """Depict pairwise MCSS alignment"""
        try:
            #
            oed = OeDepictMCSAlignPage()
            oed.setSearchType(sType="relaxed")
            #
            oed.setRefMol(oeMolRef, displayIdRef)
            oed.setFitMol(oeMolFit, displayIdFit)
            #
            # imagePath = self.__makeImagePath()
            # ---
            bondDisplayWidth = 10.0
            numAtomsRef = oeMolRef.NumAtoms()
            if numAtomsRef > 100 and numAtomsRef <= 200:
                bondDisplayWidth = 6.0
            elif numAtomsRef > 200:
                bondDisplayWidth = 4.0
            # ---
            oed.setDisplayOptions(
                imageSizeX=kwargs.get("imageSizeX", 2500),
                imageSizeY=kwargs.get("imageSizeX", 2500),
                labelAtomName=kwargs.get("labelAtomName", False),
                labelAtomCIPStereo=kwargs.get("labelAtomCIPStereo", True),
                labelAtomIndex=kwargs.get("labelAtomIndex", False),
                labelBondIndex=kwargs.get("labelBondIndex", False),
                labelBondCIPStereo=kwargs.get("labelBondCIPStereo", True),
                cellBorders=kwargs.get("cellBorders", True),
                bondDisplayWidth=bondDisplayWidth,
                highlightStyleFit=kwargs.get("highlightStyleFit", "ballAndStickInverse"),
            )
            #
            aML = oed.alignPair(imagePath=imagePath)
            logger.info("Aligned atom count %d", len(aML))
            #
            # self.assertGreater(len(aML), 1)
            # if aML:
            #    for (rCC, rAt, tCC, tAt) in aML:
            #        logger.debug("%5s %-5s %5s %-5s", rCC, rAt, tCC, tAt)
            return True
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return False

    def toMolFile(self, identifier, identifierType, molfilePath=None, fmt="mol", **kwargs):
        """Create molfile (fmt) from InChI, SMILES descriptors or PDB identifier."""
        try:
            molfilePath = molfilePath if molfilePath else self.__makeMolfilePath(fmt=fmt)
            oeio = OeIoUtils()
            if identifierType.lower() in ["smiles"]:
                oeMol = oeio.smilesToMol(identifier)
                oeMol.SetTitle("From SMILES")
            elif identifierType.lower() in ["inchi"]:
                oeMol = oeio.inchiToMol(identifier)
                oeMol.SetTitle("From InChI")
            elif identifierType.lower() in ["identifierpdb"]:
                ccsw = ChemCompSearchWrapper()
                oesmP = ccsw.getSearchMoleculeProvider()
                oeMol = oesmP.getMol(identifier)
            #
            ok = self.__toMolFile(oeMol, molfilePath, **kwargs)
            return molfilePath if ok else None
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return None

    def __toMolFile(self, oeMol, molfilePath, **kwargs):
        """Write the

        Args:
            oeMol (object): instance of an OE graph molecule
            molfilePath (string): file path for molfile (type determined by extension)

        Returns:
            bool: True for success or False otherwise
        """
        try:
            _ = kwargs
            oeio = OeIoUtils()
            oeio.write(molfilePath, oeMol, constantMol=True)
            return True
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return False

    def __makeMolfilePath(self, fmt="mol"):
        imageDirPath = self.__configD["imageDirPath"] if self.__configD and "imageDirPath" in self.__configD else "."
        fileRotateIncrement = self.__configD["fileRotateIncrement"] if self.__configD and "fileRotateIncrement" in self.__configD else 50
        ic = self.__imageCount % fileRotateIncrement
        molPath = os.path.join(imageDirPath, "molfile-%s.%s" % (ic, fmt))
        return molPath
